﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;
using Math3D;

namespace animation2
{
    public partial class _3D_Animation2 : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        double STEP = 0.3;
        double ANGLES = 1;

        uint m_legs_obj, m_body_obj, m_weapon_obj, m_head_obj;

        uint m_camera;

        string[] m_skin_files = { "..\\..\\..\\..\\models\\human_character\\skins\\skin1.jpg", "..\\..\\..\\..\\models\\human_character\\skins\\skin2.jpg", "..\\..\\..\\..\\models\\human_character\\skins\\skin3.jpg", "..\\..\\..\\..\\models\\human_character\\skins\\skin4.jpg" };


        uint m_3d_animation;

        enum ZoomMode { idle, closer, away }

        ZoomMode m_zoom_mode = ZoomMode.idle;


        public _3D_Animation2()
        {
            InitializeComponent();
        }




        class sequence_item
        {
            public string name;
            public uint seq_handle;

            public sequence_item(uint handle, string seq_name)
            {
                name = seq_name;
                seq_handle = handle;
            }

            public override string ToString()
            {
                if (name == null)
                    return ("no name");

                return name;
            }
        }









        private void OnIdle(object sender, EventArgs e)
        {

            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            zoom_camera(); //zoom the camera if the the up/down arrow buttons are clicked

            //write the yellow text on the screen
            write_frame_number("Legs sequence name=", m_legs_obj, 0, 80);
            write_frame_number("Body sequence name=", m_body_obj, 0, 90);


            if (checkBox_auto_turn.Checked)
                _3D.STATE_object_rotate_z(m_legs_obj, ANGLES, _3D.OBJECT_SPACE);


            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...


        }

        void zoom_camera()
        {
            if (m_zoom_mode == ZoomMode.idle)
                return;


            double x, y, z;
            _3D.STATE_camera_get_location(m_camera, out x, out y, out z);

            if (m_zoom_mode == ZoomMode.closer)
            {
                if (x <= 20)
                    return;

                _3D.STATE_camera_move(m_camera, _3D.WORLD_SPACE, -STEP, 0, 0);
                return;
            }

            if (x >= 200)
                return;

            _3D.STATE_camera_move(m_camera, _3D.WORLD_SPACE, STEP, 0, 0);
            return;



        }



        //The yellow text ...
        void write_frame_number(string str, uint obj, int x, int y)
        {
            string name = "error";
            int frame = -1;
            uint seq = _3D.STATE_object_get_3D_sequence(obj);
            if (seq != 0)
            {
                name = _3D.STATE_3D_sequence_get_name2(seq);
                frame = (int)_3D.STATE_3D_sequence_get_current_frame(seq);
            }

            uint ariel14 = _3D.STATE_font_create("Ariel", 14, _3D.FONT_CREATE_BOLD);
            _3D.STATE_font_write_text(ariel14, str + ": " + name + " " + frame.ToString(), x, y, 255, 255, 255, 120, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
        }

        //load a 3D object into the engine
        // These objects will be later rendered (painted on the screen) when _3D.STATE_engine_render() is called
        uint load_object(string file_name, string bmp_file, string cfg_file, string token)
        {
            uint obj;
            if (String.IsNullOrEmpty(cfg_file) == false)
                obj = _3D.STATE_object_md3_create_from_file(file_name, cfg_file, token);
            else
                obj = _3D.STATE_object_create_from_file(file_name);


            if (obj == 0)
                return (0);

            if (String.IsNullOrEmpty(bmp_file))
                return (obj);

            uint bmp = _3D.STATE_bitmap_load(bmp_file, -1);
            if (bmp == 0)
                return (obj);

            _3D.STATE_object_set_bitmap(obj, bmp, 0);
            return (obj);
        }

        //Set a specific animation sequence
        uint set_animation_sequence(uint obj, string sequence_name, bool play_backwards)
        {
            uint animation = _3D.STATE_object_get_3D_animation(obj);
            if (animation == 0)
                return (0);

            uint seq = _3D.STATE_3D_sequence_get_using_name(animation, sequence_name);
            if (seq == 0)
                return (0);

            _3D.STATE_object_set_3D_sequence(obj, seq, 0);

            _3D.STATE_3D_sequence_backwards_play(seq, play_backwards);

            return (seq);


        }

        void load_model()
        {
            m_legs_obj = load_object("..\\..\\..\\..\\models\\human_character\\Lower.md3", "..\\..\\..\\..\\models\\human_character\\skins\\skin1.jpg", "..\\..\\..\\..\\models\\human_character\\animation.cfg", "LEGS_");
            m_body_obj = load_object("..\\..\\..\\..\\models\\human_character\\Upper.md3", "..\\..\\..\\..\\models\\human_character\\skins\\skin1.jpg", "..\\..\\..\\..\\models\\human_character\\animation.cfg", "TORSO_");
            m_head_obj = load_object("..\\..\\..\\..\\models\\human_character\\Head.md3", "model\\skins\\skin1.jpg", null, null);
            m_weapon_obj = load_object("..\\..\\..\\..\\models\\human_character\\railgun.md3", null, null, null);

            //if setting scale, then set it just for the base object( legs)
            //_3D.STATE_object_set_scale(m_legs_obj, new double[3] { 3, 3, 3 });

            //connect the parts of the human model. The legs are the base so
            //for now on we will move just the base. The rest will follow automatically
            _3D.STATE_object_md3_connect_models_automatic(m_legs_obj, m_body_obj);
            _3D.STATE_object_md3_connect_models_automatic(m_body_obj, m_head_obj);
            _3D.STATE_object_md3_connect_models_automatic(m_body_obj, m_weapon_obj);

            //set the lower body to walking sequence and an upper body to the Gesture sequence
            set_animation_sequence(m_legs_obj, "LANDB", false);
            set_animation_sequence(m_body_obj, "STAND", false);

            fill_sequence_listbox(listBox_legs, m_legs_obj);
            fill_sequence_listbox(listBox_body, m_body_obj);

        }

        void fill_sequence_listbox(ListBox lb, uint obj)
        {
            if (obj == 0)
                return;

            uint anim3d = _3D.STATE_object_get_3D_animation(obj);
            if (anim3d == 0)
                return;

            lb.Items.Clear();
            for (uint seq = _3D.STATE_3D_sequence_get_first(anim3d); seq != 0; seq = _3D.STATE_3D_sequence_get_next(seq))
            {
                string name = _3D.STATE_3D_sequence_get_name2(seq);

                int num_of_frames = _3D.STATE_3D_sequence_get_number_of_frames(seq);

                sequence_item item = new sequence_item(seq, name + " (" + num_of_frames.ToString() + " frames)");

                lb.Items.Add(item);
            }
        }
        





        #region events




        private void checkBox_reverse_legs_CheckedChanged(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_legs_obj);
            if (seq == 0)
                return;


            if (checkBox_reverse_legs.Checked)
            {
                _3D.STATE_3D_sequence_backwards_play(seq, true);
                return;
            }

            _3D.STATE_3D_sequence_backwards_play(seq, false);
        }

        private void checkBox_reverse_body_CheckedChanged(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_body_obj);
            if (seq == 0)
                return;


            if (checkBox_reverse_body.Checked)
            {
                _3D.STATE_3D_sequence_backwards_play(seq, true);
                return;
            }

            _3D.STATE_3D_sequence_backwards_play(seq, false);

        }




        private void button_closer_MouseDown(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.closer;
        }

        private void button_closer_MouseUp(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.idle;
        }

        private void button_away_MouseDown(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.away;
        }

        private void button_away_MouseUp(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.idle;
        }

        private void _3D_Animation2_Load(object sender, EventArgs e)
        {
            load_model();

            _3D.STATE_engine_set_background_color(0, 0, 0);

            m_camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_location(m_camera, 50, 0, 0);
            _3D.STATE_camera_point_at(m_camera, 0, 0, 0);

            double[] direction = { 1, 1, 0 };
            uint light2 = _3D.STATE_light2_create_directional_light("my_light", direction);


            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;

        }

        private void _3D_Animation2_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;

        }

        private void numericUpDown1_legs_speed_ValueChanged(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_legs_obj);
            if (seq == 0)
                return;

            _3D.STATE_3D_sequence_set_speed(seq, (double)numericUpDown_legs_speed.Value);

        }


        private void numericUpDown_body_speed_ValueChanged(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_body_obj);
            if (seq == 0)
                return;

            _3D.STATE_3D_sequence_set_speed(seq, (double)numericUpDown_body_speed.Value);


        }

        void button_play_pause_object(Button button, uint obj, bool reverse_play)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(obj);
            if (seq == 0)
                return;

            if (_3D.STATE_3D_sequence_is_paused(seq) == _3D.YES)
            {
                _3D.STATE_3D_sequence_play(seq);
                if (reverse_play)
                    _3D.STATE_3D_sequence_backwards_play(seq, true);


                button.Text = "Pause";
                return;
            }

            _3D.STATE_3D_sequence_pause(seq);
            button.Text = "Play";



        }

        
        private void button_play_pause_legs_Click(object sender, EventArgs e)
        {
            Button button = (Button)sender;
            button_play_pause_object(button, m_legs_obj, checkBox_reverse_legs.Checked);
        }

        private void button_play_pause_body_Click(object sender, EventArgs e)
        {
            Button button = (Button)sender;
            button_play_pause_object(button, m_body_obj, checkBox_reverse_body.Checked);
        }



        private void listBox_skins_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = listBox_skins.SelectedIndex;
            string image_file=m_skin_files[index];

            uint bmp = _3D.STATE_bitmap_load(image_file, -1);
            if (bmp == 0)
                return;

            _3D.STATE_object_set_bitmap(m_legs_obj, bmp, 0);
            _3D.STATE_object_set_bitmap(m_body_obj, bmp, 0);
            _3D.STATE_object_set_bitmap(m_head_obj, bmp, 0);



        }

        private void checkBox_weapon_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_weapon.Checked)
                _3D.STATE_object_enable(m_weapon_obj);
            else
                _3D.STATE_object_disable(m_weapon_obj);

        }


        private void listBox_body_SelectedIndexChanged(object sender, EventArgs e)
        {
            sequence_item item = (sequence_item)listBox_body.SelectedItem;
            if (item == null)
                return; //safty

            _3D.STATE_object_set_3D_sequence(m_body_obj, item.seq_handle, 300);

        }


        private void listBox_legs_SelectedIndexChanged(object sender, EventArgs e)
        {
            sequence_item item = (sequence_item)listBox_legs.SelectedItem;
            if (item == null)
                return; //safty



            _3D.STATE_object_set_3D_sequence(m_legs_obj, item.seq_handle, 300);

        }

        #endregion events

        private void label5_Click(object sender, EventArgs e)
        {

        }



    }
}
