﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;
using Math3D;

namespace _3D_animation1
{
    public partial class Form1 : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        double STEP = 0.3;
        double ANGLES = 1;

        uint m_md2_object;
        uint m_skin_bmp;
        uint m_camera;

        uint m_light;

        uint m_3d_animation;

        enum ZoomMode { idle, closer, away }

        ZoomMode m_zoom_mode=ZoomMode.idle;

        
        public Form1()
        {
            InitializeComponent();
        }




        class sequence_item
        {
            public string name;
            public uint seq_handle;

            public sequence_item(uint handle, string seq_name)
            {
                name = seq_name;
                seq_handle = handle;
            }

            public override string ToString()
            {
                if (name == null)
                    return ("no name");
                
                return name;
            }
        }


        void fill_sequence_listbox()
        {
            listBox_sequences.Items.Clear();
            for (uint seq = _3D.STATE_3D_sequence_get_first(m_3d_animation); seq != 0; seq = _3D.STATE_3D_sequence_get_next(seq))
            {
                string name = _3D.STATE_3D_sequence_get_name2(seq);

                int num_of_frames = _3D.STATE_3D_sequence_get_number_of_frames(seq);

                sequence_item item = new sequence_item(seq, name + " (" + num_of_frames.ToString() + " frames)");

                listBox_sequences.Items.Add(item);
            }
        }
        
        void load_md2_object(string file_name)
        {
            if(m_md2_object!=0)
                _3D.STATE_object_delete(m_md2_object);

            m_md2_object = _3D.STATE_object_create_from_file(file_name);
            if(m_md2_object==0)
                return;//safty


            m_3d_animation = _3D.STATE_object_get_3D_animation(m_md2_object);
            if(m_3d_animation==0)
                return;//safty

            fill_sequence_listbox();

            string str=_3D.STATE_3D_animation_get_name2(m_3d_animation);
            object_set_text(m_md2_object, str);

        }


        void set_skin(string file_name)
        {
            if (m_skin_bmp != 0)
                _3D.STATE_bitmap_unload(m_skin_bmp);

           
            m_skin_bmp = _3D.STATE_bitmap_load(file_name, -1);
            _3D.STATE_object_set_bitmap(m_md2_object, m_skin_bmp, 0);

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            load_md2_object("..\\..\\..\\..\\models\\md2\\Mclain.md2");
            set_skin("..\\..\\..\\..\\models\\md2\\Mclain_3d");

            _3D.STATE_engine_set_background_color(0, 0, 0);

            m_camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_location(m_camera, 50, 0, 0);
            _3D.STATE_camera_point_at(m_camera, 0, 0, 0);


            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;

        }


        private void OnIdle(object sender, EventArgs e)
        {
            
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            zoom_camera(); //zoom the camera if the the up/down arrow buttons are clicked

            //write the yellow text on the screen
            write_frame_number("Sequence name=", m_md2_object, 0, 10);

            if (checkBox_auto_turn.Checked)
                _3D.STATE_object_rotate_z(m_md2_object, ANGLES , _3D.OBJECT_SPACE);


            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...
            

        }

        void zoom_camera()
        {
            if (m_zoom_mode == ZoomMode.idle)
                return;


            double x, y, z;
            _3D.STATE_camera_get_location(m_camera, out x, out y, out z);

            if (m_zoom_mode == ZoomMode.closer)
            {
                if (x <= 0)
                    return;

                _3D.STATE_camera_move(m_camera, _3D.WORLD_SPACE, -STEP, 0, 0);
                return;
            }

            if (x >= 200)
                return;

            _3D.STATE_camera_move(m_camera, _3D.WORLD_SPACE, STEP, 0, 0);
            return;



        }




        //Write a text above the object. This text is attached to the object
        //This function is not used in this sample
        void object_set_text(uint obj, string txt)
        {
            uint ariel14 = _3D.STATE_font_create("Ariel", 14, _3D.FONT_CREATE_BOLD);
            double[,] box = new double[2, 3];
            _3D.STATE_object_get_bounding_box(obj, box);
            double object_height = box[1, 2] - box[0, 2];
            double[] text_offset = { 0, 0, object_height * 0.75 };
            //add some text to the player
            _3D.STATE_object_add_text(obj, ariel14, txt, 255, 0, 0, _3D.FONT_WRITE_CENTERED_X | _3D.FONT_WRITE_CENTERED_Y, 3, text_offset);
        }


        //The yellow text ...
        void write_frame_number(string str, uint obj, int x, int y)
        {
            string name = "error";
            int frame = -1;
            uint seq = _3D.STATE_object_get_3D_sequence(obj);
            if (seq != 0)
            {
                name = _3D.STATE_3D_sequence_get_name2(seq);
                frame = (int)_3D.STATE_3D_sequence_get_current_frame(seq);
            }

            uint ariel14 = _3D.STATE_font_create("Ariel", 14, _3D.FONT_CREATE_BOLD);
            _3D.STATE_font_write_text(ariel14, str + ": " + name + " " + frame.ToString(), x, y, 255, 255, 255, 120, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
        }

        private void button_load_object_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();

            string dir = Directory.GetCurrentDirectory();
            
            dlg.InitialDirectory = "..\\..\\..\\..\\models\\md2\\";
            dlg.Filter = "Md2 files (*.md2)|*.md2|All files (*.*)|*.*";
            
            DialogResult res=dlg.ShowDialog();
            if (res != DialogResult.OK)
                return;

            if (String.IsNullOrEmpty(dlg.FileName))
                return;

            load_md2_object(dlg.FileName);

        }

        private void button_load_skin_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();

            dlg.InitialDirectory = "..\\..\\..\\..\\models\\md2\\";
            dlg.Filter = "bmp files (*.bmp)|*.bmp|All files (*.*)|*.*";

            DialogResult res = dlg.ShowDialog();
            if (res != DialogResult.OK)
                return;

            if (String.IsNullOrEmpty(dlg.FileName))
                return;


            set_skin(dlg.FileName);


        }

        private void listBox_sequences_SelectedIndexChanged(object sender, EventArgs e)
        {
            sequence_item item = (sequence_item)listBox_sequences.SelectedItem;

            _3D.STATE_object_set_3D_sequence(m_md2_object, item.seq_handle, 1000);
        }

        private void button_play_pause_Click(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_md2_object);
            if (seq == 0)
                return;

            if (_3D.STATE_3D_sequence_is_paused(seq) == _3D.YES)
            {
                _3D.STATE_3D_sequence_play(seq);
                if (checkBox_reverse.Checked)
                    _3D.STATE_3D_sequence_backwards_play(seq, true);


                button_play_pause.Text = "Pause";
                return;
            }

            _3D.STATE_3D_sequence_pause(seq);
            button_play_pause.Text = "Play";

        }

        private void checkBox_reverse_CheckedChanged(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_md2_object);
            if (seq == 0)
                return;


            if (checkBox_reverse.Checked)
            {
                _3D.STATE_3D_sequence_backwards_play(seq, true);
                return;
            }

            _3D.STATE_3D_sequence_backwards_play(seq, false);
            

        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            uint seq = _3D.STATE_object_get_3D_sequence(m_md2_object);
            if (seq == 0)
                return;

            _3D.STATE_3D_sequence_set_speed(seq, (double)numericUpDown1.Value);

        }


        private void button_closer_MouseDown(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.closer;
        }

        private void button_closer_MouseUp(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.idle;
        }

        private void button_away_MouseDown(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.away;
        }

        private void button_away_MouseUp(object sender, MouseEventArgs e)
        {
            m_zoom_mode = ZoomMode.idle;
        }

        void create_light()
        {
            double[] direction = { -1, -1, -1 };
            m_light = _3D.STATE_light2_create_directional_light("my_light", direction);
        }

        private void checkBox_light_CheckedChanged(object sender, EventArgs e)
        {

            if (checkBox_light.Checked)
            {
                if (m_light == 0)
                    create_light();

                _3D.STATE_light2_disable_all_lighting(false);

                
                _3D.STATE_light2_enable(m_light);
                return;
            }

            //_3D.STATE_light2_disable(m_light);
            _3D.STATE_light2_disable_all_lighting(true);


        }

 
    }
}
