﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;
using Math3D;

/*
 * 
 *  Deal with the 3D text we have before each level starts
 * 
 
 */ 

namespace OneAgainst100Planes
{


    public class Text3D
    {
        List<uint> m_text3d_objects;
        double[] m_3d_text_rotation_center;
        double m_3d_text_turning_radius = 200000;

        //convert the given strings into 3D objects that can be displayed in 3D
        //the 3D text objects will be placed in a circle which its center is a point above the location of the camera
        //so that the bottom of the circle is the camera location
        public Text3D(string[] texts, uint font)
        {
            m_text3d_objects = new List<uint>(texts.Length);
            
            uint camera = _3D.STATE_camera_get_default_camera();
            
            double[] camera_location = new double[3];
            _3D.STATE_camera_get_location1(camera, camera_location);

            m_3d_text_rotation_center = Vector.copy(camera_location);
            m_3d_text_rotation_center[2] += m_3d_text_turning_radius;


            double angle_gap = 270 / texts.Length;
            double angle = angle_gap;

            foreach (string str in texts)
            {
                uint obj = _3D.STATE_object_create_from_text(font, str, 255, 128, 128, _3D.FONT_WRITE_DEFAULT | _3D.FONT_WRITE_CENTERED_X | _3D.FONT_WRITE_CENTERED_Y);
                _3D.STATE_object_set_location1(obj, camera_location);
                _3D.STATE_object_rotate_around_point(obj, -angle, _3D.Y_AXIS, m_3d_text_rotation_center);
                angle += angle_gap;
                double[] scale = { 1000, 1000, 1000 };
                _3D.STATE_object_set_scale(obj, scale);
                m_text3d_objects.Add(obj);
            }



        }

        //since the camera might have moved from the time we created the 3d texts
        //we need to update the location of the 3d texts
        //
        // The reason the camera is moving is because it follows the plane.
        //we can avoid this function if we set the camera explicitly to direction 1,0,0 and cancel chasing the airplane
        //(though this solution wont look so nice since the camera will move ubruptly)
        void update_text_objects_location_according_to_camera()
        {

            uint camera = _3D.STATE_camera_get_default_camera();
            
            double[] camera_location = new double[3];
            _3D.STATE_camera_get_location1(camera, camera_location);

            double[] new_text_rotation_center;
            new_text_rotation_center = Vector.copy(camera_location);
            new_text_rotation_center[2] += m_3d_text_turning_radius;

            double[] diff = Vector.copy(new_text_rotation_center);
            Vector.sub_vec(diff, m_3d_text_rotation_center);

            foreach (uint obj in m_text3d_objects)
                _3D.STATE_object_move(obj, _3D.WORLD_SPACE, diff[0], diff[1], diff[2]);

            Vector.copy(m_3d_text_rotation_center, new_text_rotation_center);

    }
        
        
        public void turn_text(double angle)
        {
            update_text_objects_location_according_to_camera();
            
            foreach (uint obj in m_text3d_objects)
                _3D.STATE_object_rotate_around_point(obj, angle, _3D.Y_AXIS, m_3d_text_rotation_center);

        }

        public void clear()
        {
            foreach (uint obj in m_text3d_objects)
                _3D.STATE_object_delete(obj);

            m_text3d_objects.Clear();
        }

    }

}