﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;
using Math3D;


namespace OneAgainst100Planes
{
    public partial class Game : Form
    {
        List<uint> m_enemy_missiles = new List<uint>();

        DateTime m_last_time_enemy_fired;
        //double m_enemy_firing_delay = 0.5; //every 0.5 seconds the enemy is allowed to fire

        const int m_number_of_enemies_shooting_simultaneously=5;

        uint[] m_last_enenmies_shootings=new uint[m_number_of_enemies_shooting_simultaneously];


        uint add_enemy_plane(double[] offset, Random r)
        {

            uint plane = object_create_copy(m_current_level.enemy_plane.object_handle, _3D.NO);

            //this function do nothing. it is just for our own use
            _3D.STATE_object_set_object_type_number(plane, (int)ObjectType.Plane);
            _3D.STATE_object_set_control_type_number(plane, (int)ObjectOwner.Computer);


            _3D.STATE_object_set_chase_type(plane, _3D.CHASE_TRACK_AIRPLANE);
            _3D.STATE_object_set_track(plane, m_track1);

            _3D.STATE_object_set_track_offset(plane, offset);
            _3D.STATE_object_set_absolute_speed(plane, m_current_level.enemy_speed);
            //_3D.STATE_object_set_absolute_speed(plane, 1);

            int num_of_planes = m_current_level.number_of_enemy_planes;
            if ((num_of_planes <= 5) && (checkBox_shadows.Checked)) //when having many planes it will slow down rendering
                _3D.STATE_object_auto_add_shadows(plane, _3D.ON, 1);

            //lets move the enemy object abit so it is not on the same spot as out own plane
            _3D.STATE_object_move(plane, _3D.OBJECT_SPACE, r.Next(50000, 500000), 0, 50000);

            if (m_current_level.enemy_AI != AI.None)
                enemy_create_own_track(plane); //since in level 2 and up we change the track points in order to give some wizdom to the enemy planes
            //We need that every plane will have its own track (in level one, only one track is used for all palnes, each time with different offset)


            m_enemy_planes.Add(plane);

            return (plane);

        }

        double get_a_random_location(int index, Random r)
        {
            double min_value = m_world_bounding_box[0, index] * 0.5;
            double max_value = m_world_bounding_box[1, index] * 0.5;

            double full_length = max_value - min_value;

            double n = r.NextDouble(); //n is between 0 to 1

            //transfer the 0-1 scope to min_value-max_value scope
            return (n * full_length + min_value);
        }

        void create_enemy_objects()
        {
            m_track1 = _3D.STATE_track_get_using_name("track1");


            //m_world_bounding_box
            Random r = new Random(DateTime.Now.Second);
            double x, y, z;

            int num_of_planes = m_current_level.number_of_enemy_planes;
            int num_of_tanks = m_current_level.number_of_enemy_tanks;

            for (int i = 0; i < num_of_planes; i++)
            {

                x = get_a_random_location(0, r);
                y = get_a_random_location(1, r);
                z = 70000 + r.Next(-20000, 20000);

                double[] offset = { x, y, z };
                add_enemy_plane(offset, r);
            }

            for (int i = 0; i < num_of_tanks; i++)
            {
                x = get_a_random_location(0, r);
                y = get_a_random_location(1, r);
                z = 70000 + r.Next(-20000, 20000);

                double[] offset2 = { x, y, z };
                add_enemy_tank(offset2);
            }

            //now set the sounds
            m_current_level.enemy_plane.load_sounds(m_enemy_planes[0]); //m_enemy_planes[0] is not important because later we attach each sound to one of the closest enemies

        }

        uint add_enemy_tank(double[] offset)
        {
            uint tank = object_create_copy(m_current_level.tank.object_handle, _3D.NO);
            _3D.STATE_object_set_object_type_number(tank, (int)ObjectType.Tank);
            _3D.STATE_object_set_control_type_number(tank, (int)ObjectOwner.Computer);


            _3D.STATE_object_set_chase_type(tank, _3D.CHASE_TRACK);
            _3D.STATE_object_set_track(tank, m_track1);

            _3D.STATE_object_set_track_offset(tank, offset);
            _3D.STATE_object_set_absolute_speed(tank, m_plane_speed / 4);

            //lets move the enemy object abit so it is not on the same spot as out own plane
            Random r = new Random();
            _3D.STATE_object_move(tank, _3D.OBJECT_SPACE, r.Next(50000, 500000), 0, 0);



            double height, width, length;
            _3D.STATE_object_get_dimensions(tank, out length, out width, out height);
            _3D.STATE_object_set_falling_from_track(tank, height / 2, _3D.YES);

            m_enemy_tanks.Add(tank);
            return (tank);

        }



        void manage_enemy_missiles()
        {
            enemies_shoot_missile();

            if(checkBox_practice_mode.Checked==false)
                check_whether_a_missile_hit_the_player();
            
            check_whether_a_bomb_hit_terrain(m_enemy_missiles);

            set_sound_to_near_objects(m_enemy_missiles, m_enemy_missile_sounds, ObjectType.Missile);
            
        }

        public class object_distance
        {
            public uint obj;
            public double distance;
        }

        double[] get_enemy_shooting_direction(uint enemy_object, EnemyShootingStyle shooting_style)
        {
            if (shooting_style == EnemyShootingStyle.Down)
            {
                double[] direction_down = { 0, 0, -1 };
                return (direction_down);
            }

            if (shooting_style == EnemyShootingStyle.Up)
            {
                double[] direction_up = { 0, 0, 1 };
                return (direction_up);
            }


            if (shooting_style == EnemyShootingStyle.Straight)
            {
                double[] direction = new double[3];
                _3D.STATE_object_get_direction1(enemy_object, direction);
                return (direction);
            }

            if (shooting_style == EnemyShootingStyle.Random)
            {
                int rnd_shooting_style=m_random.Next(1, 3);
                return(get_enemy_shooting_direction(enemy_object, (EnemyShootingStyle)rnd_shooting_style)); //calling the same function with one of the first 3 shooting styles (Up,Down or Straight)
            }

            //from here on shooting style must be AimOnPlayer
            double[] player_location = new double[3];
            _3D.STATE_object_get_location1(m_current_level.player_plane.object_handle, player_location);

            double[] enemy_location = new double[3];
            _3D.STATE_object_get_location1(enemy_object, enemy_location);
            Vector.sub_vec(player_location, enemy_location); //player_location=player_location-enemy_location

            _3D.STATE_math_normalize_vector(player_location);
            return (player_location); //now player_location holds the direction from the enemy to the players plane

        }

        


        bool is_player_in_front_of_enemy_shooting_direction(uint enemy_object, double[] enemy_location, double[] enemy_shooting_direction,double[] player_location)
        {

            double[] plane_equation = new double[4]; //Plane Equasion is Ax+By+Cz+D=0
            Vector.copy(plane_equation, enemy_shooting_direction);

            //now enemy_direction has the plane equation. We will now set the d parameter of the plane equation
            plane_equation[3] = -1 * _3D.STATE_math_product(enemy_location, enemy_shooting_direction);

            int is_player_in_front = _3D.STATE_math_get_plane_point_relations(plane_equation, player_location, 0);

            if (is_player_in_front != _3D.IN_FRONT_OF_PLANE)
                return (false);


            return (true);


        }

        void enemies_shoot_missile()
        {


            DateTime now = DateTime.Now;
            TimeSpan pause_time = now - m_last_time_enemy_fired;
            if (pause_time.TotalMilliseconds < m_current_level.enemy_firing_delay)
                return;

            double[] player_location = new double[3];
            _3D.STATE_object_get_location1(m_plane_object, player_location);

            double min_distance = 1e100;
            uint enemy_to_shoot = 0;

            foreach (uint enemy in m_enemy_planes)
            {
                ObjectType obj_type = (ObjectType)_3D.STATE_object_get_object_type_number(enemy);


                if (did_this_enemy_shoot_recently(enemy))
                    continue;


                double[] enemy_location=new double[3];
                _3D.STATE_object_get_location1(enemy, enemy_location);

                double[] enemy_shooting_direction = get_enemy_shooting_direction(enemy, m_current_level.enemy_shooting_style);

                if (is_player_in_front_of_enemy_shooting_direction(enemy, enemy_location, enemy_shooting_direction, player_location) == false)
                    continue;

                /*
                _3D.STATE_object_move(enemy, _3D.OBJECT_SPACE, -100, 0,0); //move forward 100 units
                
                double[] enemy_location_forward=new double[3];
                _3D.STATE_object_get_location1(enemy, enemy_location_forward);

                //get the enemy back to its previous position
                _3D.STATE_object_move(enemy, _3D.OBJECT_SPACE, 100, 0,0); 
                */

                double[] enemy_location_forward = new double[3];
                _3D.STATE_object_get_location1(enemy, enemy_location_forward);
                Vector.add_vec(enemy_location_forward, enemy_shooting_direction);

                double trajectory_distance_from_target = _3D.STATE_math_get_distance_between_point_and_line(player_location, enemy_location, enemy_location_forward);

                if (trajectory_distance_from_target < min_distance)
                {
                    enemy_to_shoot = enemy;
                    min_distance = trajectory_distance_from_target;
                }

            
            }

            if (enemy_to_shoot == 0)
                return;



            uint missile = launch_missile(enemy_to_shoot, false);
            if (missile == 0)
                return;

            m_last_time_enemy_fired = now;

            m_enemy_missiles.Add(missile);

            store_last_enenmy_who_fired(enemy_to_shoot);
            


        }


        void check_whether_a_missile_hit_the_player()
        {
            if (m_game_status != GameStatus.Playing)
                return;
            
            double[] player_location = new double[3];
            _3D.STATE_object_get_location1(m_plane_object, player_location);

            double killing_zone_sq=m_missile_killing_zone*m_missile_killing_zone;
            
            foreach (uint missile in m_enemy_missiles)
            {
                double[] missile_location = new double[3];
                _3D.STATE_object_get_location1(missile, missile_location);

                double square_distance=_3D.STATE_math_get_square_distance(player_location, missile_location);

                if (square_distance > killing_zone_sq)
                    continue;

                m_game_status = GameStatus.Player_Was_Killed;
                kill_player_plane(m_plane_object);

                _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_FLEXIBLE);

                return;
                //double[] offset = { 0, 0, 15000 }; //set an offset so it is always a bit above
                //_3D.STATE_camera_set_chase_offset(m_camera, offset);
            
            }

        }

        //we store the last enemy shot in m_last_enenmies_shootings[0] 
        //so every time before we store the last one, we shift the previous shooters one place up
        void store_last_enenmy_who_fired(uint enemy)
        {
            for (int i = 0; i < m_number_of_enemies_shooting_simultaneously-1; i++)
                m_last_enenmies_shootings[i+1] = m_last_enenmies_shootings[i];


            m_last_enenmies_shootings[0] = enemy;
        }


        bool did_this_enemy_shoot_recently(uint enemy)
        {
            //note that if only one enemy is left, then he has too shoot all the time
            //if two are left then we want both of them firing in turns and so on
            
            int how_many_to_check = m_enemy_planes.Count/2;
            if (how_many_to_check > m_number_of_enemies_shooting_simultaneously)
                how_many_to_check = m_number_of_enemies_shooting_simultaneously;


            for (int i = 0; i < how_many_to_check; i++)
                if (m_last_enenmies_shootings[i] == enemy)
                    return(true);


            return (false);

        }



        void delete_all_dead_enemies()
        {
            foreach (uint obj in m_killed_enemies)
                delete_object(obj);

            m_killed_enemies.Clear();

        }

        void restart_game()
        {
            if(m_current_level!=null)
                m_current_level.disable_level_objects(); //disable the player plane model from the previous level

            m_game_status = GameStatus.MainMenu;
            m_duration_of_last_finished_level = 360000;

            m_starting_level = show_main_menu_dialog();
            m_game_level = m_starting_level;

            delete_all_dead_enemies();

            m_game_status = GameStatus.Level_Is_Finished;
            show_game_gui_controls(true);
            create_3d_texts();
            m_levels[0].player_plane.load(false); //disable it so it is not seen while we have the 3D text

            make_the_camera_follow_the_airplane_automatically();

        }
        
        void check_that_killed_player_plane_hit_the_ground()
        {
            check_that_the_camera_doesnt_go_under_the_ground();
            
            double height_above_ground = _3D.STATE_object_get_center_height_above_ground(m_plane_object, _3D.YES);
            if (height_above_ground > 1000)
                return;

            _3D.STATE_object_set_chase_type(m_plane_object, _3D.NO_CHASE);

            _3D.STATE_camera_set_chase_type(m_camera, _3D.NO_CHASE);



            m_game_status = GameStatus.GameOver;
            show_game_gui_controls(false);


            explode_plane(m_plane_object);
            play_sound(m_explosion_sound, 0);



        }

        void delete_object(uint obj)
        {
           
            _3D.STATE_object_delete(obj);
            m_total_number_of_object_created--;

            m_allocated_3d_objects_dict.Remove(obj);
        }


        //delete 4 things:
        // enemy planes, enemy tanks, enemy bombs and missiles, player bobmbs and missiles
        void delete_enemies_and_bombs()
        {
            foreach (uint bomb in m_bombs)
                delete_object(bomb);

            m_bombs.Clear();



            foreach (uint missile in m_enemy_missiles)
                delete_object(missile);

            m_enemy_missiles.Clear();

            foreach (uint enemy in m_enemy_planes)
                delete_object(enemy);

            m_enemy_planes.Clear();

            foreach (uint enemy in m_enemy_tanks)
                delete_object(enemy);

            m_enemy_tanks.Clear();



        }


    }

}