﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using System.Web;

using _3dstate;
//using Math3D;


namespace OneAgainst100Planes
{
    public partial class Game : Form
    {

        List<uint> m_bombs = new List<uint>();

        //also include the explosions. We delete these 3dstate objects when we start a new level
        List<uint> m_killed_enemies = new List<uint>();

        enum ObjectOwner { Unknown = 0, Me = 1, Computer = 2 }

        Text3D m_3d_text; //the text that appers before each level


        //old not in use
        void show_help_popup()
        {
            string str = File.ReadAllText("..\\..\\..\\..\\Docs\\Airplane2_help.txt");

            if (String.IsNullOrEmpty(str))
                str = "couldnt find help file Networking5_help.txt";

            MessageBox.Show(str, "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);



            //MessageBox.Show("Instructions:\n\nPress F1 To get this help popup\n\nUse the arrow keys to navigate the plane\n\nUse Q,W,E to change the camera modes\n\nUse 1,2,3,4,5 to control the camera distance\n\nUse A,S to toggle shadow modes\n\nUse D,F,G,H to toggle sound modes\n\nUse J,K to control fog intensity\n\nUse the space bar and 'B' to increase or decrease the speed\n\nThings to notice\n------------------\n1. Press E (NO_CHASE) and notice how the sound gets automatically stronger when the plane gets closer to you\nNotice that when the plane is on the right, you hear it more from the right speaker\n(make sure your speaker are positioned correctly)\n\n2. Press W and then 2 and notice the shadows on the plane's wing and tail (when turning)\n", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }




        void create_light()
        {
            double[] direction = { -1, -1, -1 };
            m_light = _3D.STATE_light2_create_directional_light("my_light", direction);
        }

        void add_some_light()
        {
            add_some_light_realtime();
            add_some_light_software();
        }
        
        //realtime light using hardware excellaration
        void add_some_light_realtime()
        {
            create_light();
        }

        //one time light calculation for the entire terrain using software calculations
        void add_some_light_software()
        {
            //Add a light source
            double[] location = { 0, 0, 10000 }; //put it up in the sky like the sun
            uint light = _3D.STATE_light_create("light1", location);
            _3D.STATE_light_set_type_of_light(light, _3D.LIGHT_AMBIENT | _3D.LIGHT_DIFFUSE);
            _3D.STATE_light_set_ambient(light, 0.8);
            _3D.STATE_light_set_specular(light, 0);
            _3D.STATE_light_set_diffuse(light, 3);
            _3D.STATE_light_activate(light, _3D.LIGHT_SET);

            location[2] = 10000000; //this height will look better with the plane shadows since is simulate the sun and the sun is very high above ...
            _3D.STATE_light_set_location(light, location);


        }


        void make_the_camera_follow_the_airplane_automatically()
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_set_chase_type(camera, _3D.CHASE_PRECISE);
            _3D.STATE_camera_set_object_to_chase(camera, m_plane_object);
            _3D.STATE_camera_set_chase_distance(camera, (double)numericUpDown_chase_distance.Value); //set the camera behind the airplane
            double[] offset = { 0, 0, 10000 }; //set an offset so it is always a bit above
            _3D.STATE_camera_set_chase_offset(camera, offset);
        }



        void check_that_the_camera_doesnt_go_under_the_ground()
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            double height;
            uint ground_poly = _3D.STATE_camera_get_height_above_ground(camera, 0, out height);
            if (ground_poly != 0)
            {
                if (height < m_camera_min_height)
                {
                    _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, m_camera_min_height - height);
                }
                return;
            }

            //the camera is under the ground ! we need to move it up
            double[] camera_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, camera_location);

            double[] up_in_the_sky = new double[3];
            up_in_the_sky[0]=camera_location[0];
            up_in_the_sky[1]=camera_location[1];
            up_in_the_sky[2]=camera_location[2]+ 1e10;

            uint obj;
            double[] intersection=new double[3];
            int no_ground_above=_3D.STATE_engine_is_movement_possible(camera_location, up_in_the_sky, out ground_poly, intersection, out obj);

            if (no_ground_above == _3D.NO)
            {
                intersection[2] += m_camera_min_height;
                _3D.STATE_camera_set_location1(m_camera, intersection);
                return;
            }


            //we should never get here. it means no ground below and no ground above !

        }


        

        public static uint object_create_copy(uint source_obj, int duplicate_polygons)
        {
            uint obj = _3D.STATE_object_duplicate(source_obj, duplicate_polygons);
            m_total_number_of_object_created++;
            
            //for debugging
            string name;
            _3D.STATE_object_get_name(obj, out name);
            m_allocated_3d_objects_dict.Add(obj, name);
            return (obj);
        }

        void explode_bomb(uint bomb)
        {
            uint explosion_master_copy = m_levels[m_game_level].explosion.object_handle;

            uint explosion = object_create_copy(explosion_master_copy, _3D.NO);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(bomb, location);

            _3D.STATE_object_set_location1(explosion, location);

            double length, width, height;
            _3D.STATE_object_get_dimensions(explosion, out length, out width, out height);
            _3D.STATE_object_move(explosion, _3D.WORLD_SPACE, 0, 0, height / 2);


            delete_object(bomb);

            _3D.STATE_object_set_event(explosion, 5000, _3D.STATE_DELETE);
            m_total_number_of_object_created--;
            m_allocated_3d_objects_dict.Remove(explosion);

            play_sound(m_explosion_sound, explosion);

        }


        uint explode_plane(uint plane)
        {
            uint explosion_master_copy = m_levels[m_game_level].explosion.object_handle;
            uint explosion = object_create_copy(explosion_master_copy, _3D.NO);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(plane, location);

            _3D.STATE_object_set_location1(explosion, location);

            //keep it burning
            //_3D.STATE_object_set_event(explosion, 5000, _3D.STATE_DELETE);

            m_killed_enemies.Add(explosion); //so that we also delete the explosion when a new level begins.

            play_sound(m_explosion_sound, explosion);

            return (explosion);

        }


        double get_bomb_killing_zone(uint bomb)
        {
            ObjectType bomb_type = (ObjectType)_3D.STATE_object_get_object_type_number(bomb);

            if (bomb_type == ObjectType.Bomb)
                return (m_bomb_killing_zone);

            return (m_missile_killing_zone);
        }

        uint did_bomb_hit_enemy(uint bomb)
        {

            double killing_zone2 = get_bomb_killing_zone(bomb);
            killing_zone2 = killing_zone2 * killing_zone2; //we use square distance so to avoid doing root calculation (a^2 + b^2 = C^2)

            double[] bomb_location = new double[3];
            _3D.STATE_object_get_location1(bomb, bomb_location);

            double[] enemy_location = new double[3];

            foreach (uint enemy in m_enemy_planes)
            {
                    _3D.STATE_object_get_location1(enemy, enemy_location);
                    double square_distance = _3D.STATE_math_get_square_distance(enemy_location, bomb_location);

                    if (square_distance > killing_zone2)
                        continue;

                    return(enemy);
            }

            foreach (uint enemy in m_enemy_tanks)
            {
                _3D.STATE_object_get_location1(enemy, enemy_location);
                double square_distance = _3D.STATE_math_get_square_distance(enemy_location, bomb_location);

                if (square_distance > killing_zone2)
                    continue;

                return (enemy);
            }

            

            return (0);

        }


        void kill_player_plane(uint plane_got_hit)
        {
            double[] direction = new double[3];
            _3D.STATE_object_get_direction1(plane_got_hit, direction);

            _3D.STATE_object_set_chase_type(plane_got_hit, _3D.CHASE_PHYSICS);
            _3D.STATE_object_set_speed_units(_3D.UNITS_PER_SECOND);
            _3D.STATE_object_cancel_collision_test_for_chase_physics(plane_got_hit);

            double[] force = { 0, 0, -200 }; //make a much slower falling for the player airplane
            _3D.STATE_object_set_force(plane_got_hit, force);


            _3D.STATE_object_set_speed_direction(plane_got_hit, direction);


            _3D.STATE_object_set_absolute_speed(plane_got_hit, m_plane_speed);

            _3D.STATE_object_set_physics_rotation(plane_got_hit, _3D.OBJECT_SPACE, 90, 30, 0);

            
            play_sound(m_explosion_sound, 0);


            _3D.STATE_sound_play(m_pain_sound, _3D.SOUND_NO_LOOP);

            


        }
        
        void kill_plane(uint plane_got_hit)
        {

            m_current_level.enemy_plane.play_death_sound(plane_got_hit);
            
            
            _3D.STATE_object_set_chase_type(plane_got_hit, _3D.CHASE_PHYSICS);

            _3D.STATE_object_set_speed_units(_3D.UNITS_PER_SECOND);

             double[] force = { 0, 0, -200 };
             _3D.STATE_object_set_force(plane_got_hit, force);

            double[] direction = new double[3];
            _3D.STATE_object_get_direction1(plane_got_hit, direction);

            _3D.STATE_object_set_speed_direction(plane_got_hit, direction);


            _3D.STATE_object_set_absolute_speed(plane_got_hit, m_current_level.enemy_speed);

            _3D.STATE_object_set_physics_rotation(plane_got_hit, _3D.OBJECT_SPACE, 30, 9, 0);


            _3D.STATE_object_cancel_collision_test_for_chase_physics(plane_got_hit);

            _3D.STATE_object_set_object_type_number(plane_got_hit, (int)ObjectType.Killed);

            uint sound = _3D.STATE_sound_get_handle_using_entity(plane_got_hit);
            if (sound != 0)
                _3D.STATE_sound_stop(sound);



            int hits = Convert.ToInt32(textBox_plane_hits.Text);
            hits++;
            textBox_plane_hits.Text = hits.ToString();

            check_qualification_for_next_level();

        }

        void check_qualification_for_next_level()
        {
            if (m_game_status == GameStatus.Level_Is_Finished)
                return; //already qualified ...

            
            
            int enemy_planes_destroyed = Convert.ToInt32(textBox_plane_hits.Text);
            int enemy_tanks_destroyed = Convert.ToInt32(textBox_tank_hits.Text);

            if (enemy_planes_destroyed < m_current_level.number_of_enemy_planes_to_kill)
                return;

            if (enemy_tanks_destroyed < m_current_level.number_of_enemy_tanks_to_kill)
                return;

            level_was_finished();

        }

        int m_duration_of_last_finished_level = 360000;//time in milliseconds. 360000 == 100 hours ...

        void level_was_finished()
        {
            m_game_status = GameStatus.Level_Is_Finished;

            TimeSpan time_passed=m_current_level.get_time_passed();

            m_duration_of_last_finished_level = (int)time_passed.TotalMilliseconds;

            MainMenu mm = new MainMenu();

            m_current_level.disable_level_objects();

            if (m_game_level >= m_levels.Length - 1)
            {
                MessageBox.Show("Bravo, You finished the game. Unbelievable !!!");
                m_game_was_completed = true;
                m_game_status = GameStatus.GameOver;
                show_game_gui_controls(false);

                return;
            }

            if (checkBox_practice_mode.Checked)
            {
                if (mm.m_max_level_index <= m_game_level) //we have finished a level that we never finished without practice mode
                {                                         //so in this case we cannot advance to the next level. We do the same level again but this time not in practice mode
                    checkBox_practice_mode.Checked = false;
                    m_game_level--;//cancel the going to the next level below ...
                }
            }

            m_game_level++;

            //initialize_next_level_3D_text();
            create_3d_texts();


        }

        void create_3d_texts()
        {
            if (m_game_level  >= m_levels.Length)
                return; //for safty. if there are no more levels. should never happen

            Level level = m_levels[m_game_level ];

            string[] texts = level.level_text.Split('\n');
            m_3d_text = new Text3D(texts, m_ariel_title);

            //note that the camera will follow the plane orientation
            //since we will turn the text object around the WORLD Y axis 
            //it is important that the camera will point towards the X axis system.
            _3D.STATE_object_set_bank(m_plane_object, 0); //set the plane to be horizontal
            _3D.STATE_object_set_direction(m_plane_object, 1, 0, 0);

        }

        //player object or computers object ?
        bool is_it_my_object(uint bomb)
        {
            ObjectOwner owner = (ObjectOwner)_3D.STATE_object_get_control_type_number(bomb);

            if (owner == ObjectOwner.Me)
                return (true);


            return (false);



        }

        void deal_with_killed__enemy_airplanes_reaching_the_ground()
        {
            List<uint> obj_to_remove = new List<uint>();
            
            foreach (uint obj in m_enemy_planes)
            {
                ObjectType ot = (ObjectType)_3D.STATE_object_get_object_type_number(obj);

                if (ot != ObjectType.Killed)
                    continue;



                double height=_3D.STATE_object_get_center_height_above_ground(obj, _3D.YES);

                if (height > 5000) 
                    continue;

                _3D.STATE_object_set_chase_type(obj, _3D.NO_CHASE);

                explode_plane(obj);

                obj_to_remove.Add(obj);
                m_killed_enemies.Add(obj);

            }

            foreach(uint obj2 in obj_to_remove)
                m_enemy_planes.Remove(obj2);


        }

        void kill_tank(uint tank)
        {
            uint tank_destroyed_master_copy = m_levels[m_game_level].tank_destroyed.object_handle;

            uint tank_x1 = object_create_copy(tank_destroyed_master_copy, _3D.NO);

            double[] location = new double[3];
            double[] direction = new double[3];

            _3D.STATE_object_get_location1(tank, location);
            _3D.STATE_object_get_direction1(tank, direction);

            _3D.STATE_object_set_location1(tank_x1, location);
            _3D.STATE_object_set_direction1(tank_x1, direction);

            m_enemy_tanks.Remove(tank);

            int hits = Convert.ToInt32(textBox_tank_hits.Text);
            hits++;
            textBox_tank_hits.Text = hits.ToString();


            delete_object(tank);
            m_killed_enemies.Add(tank_x1);


            check_qualification_for_next_level();


        }

        void kill_object(uint obj_got_hit)
        {
            ObjectType ot= (ObjectType)_3D.STATE_object_get_object_type_number(obj_got_hit);
            if (ot == ObjectType.Plane)
            {
                kill_plane(obj_got_hit);
                string text = "Target Destroyed.   Well Done !";
                if( (m_game_status==GameStatus.Player_Was_Killed) || (m_game_status==GameStatus.GameOver))
                    text = "Target Destroyed.   Too late for you ..."; //in case one of our missiles hit the target after we are dead ...

                _3D.STATE_font_write_text(m_ariel_title, text, 50, 25, 255, 60, 160, 60, _3D.FONT_WRITE_CENTERED_X, 100);
                return;
            }

            if (ot == ObjectType.Tank)
            {
                kill_tank(obj_got_hit);
                _3D.STATE_font_write_text(m_ariel_title, "Tank Destroyed.   Good Job !", 50, 25, 255, 60, 60, 160, _3D.FONT_WRITE_CENTERED_X, 100);

                return;
            }

        }

        void check_whether_a_bomb_hit_enemy()
        {
            if (m_game_status != GameStatus.Playing)
                return;

            
            List<uint> bombs_to_delete = new List<uint>();
            List<uint> bombs_to_add = new List<uint>();

            foreach (uint bomb in m_bombs)
            {
                uint obj_got_hit = did_bomb_hit_enemy(bomb);
                if (obj_got_hit == 0)
                    continue;

                explode_bomb(bomb);
                bombs_to_delete.Add(bomb);
                kill_object(obj_got_hit);
                _3D.STATE_sound_stop(m_player_missile_sound);

            }

            foreach (uint bomb in bombs_to_delete)
                m_bombs.Remove(bomb);

            foreach (uint bomb in bombs_to_add)
                m_bombs.Add(bomb);


        }



        //handle missile and bombs
        void check_whether_a_bomb_hit_terrain(List<uint> bomb_list)
        {
            List<uint> bombs_to_delete = new List<uint>();

            foreach (uint bomb in bomb_list)
            {
                double[] location = new double[3];
                _3D.STATE_object_get_location1(bomb, location);

                if (is_point_inside_the_world(location) == false)
                {
                    //the missile went outside the world !
                    delete_object(bomb); //notice that we delete the 3D object inside the 3d engine. it is not that we are delting something inside the foreach loop ...
                    bombs_to_delete.Add(bomb);
                    continue;
                }

                if (location[2] > m_highest_mountain_top)
                    continue; //above the highest mountain so no need to check collision

                //checking the collision of the bomb with the terrain is expensive and takes time
                //it is also not so important that we discover that the bomb is underground immediatelly when it happens
                //so for a specific bomb we check it collision every 4 renders (in average)
                //if it is underground we will discover it prety quickly and since the explosion is big we will almost always see it.
                if (m_random.Next(1, 4) == 1)
                {
                    //note that STATE_object_get_center_height_above_ground(0 does collision detection so it is an expensive function
                    double height_above_ground = _3D.STATE_object_get_center_height_above_ground(bomb, _3D.YES);
                    if (height_above_ground > 0)
                        continue;

                    explode_bomb(bomb);

                    bombs_to_delete.Add(bomb);
                }

            }

            foreach (uint bomb in bombs_to_delete)
                bomb_list.Remove(bomb);


        }


        void check_whether_a_bomb_is_fired()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.M) >= 0)
                return;

            fire_a_bomb(m_plane_object, m_plane_speed * 1.3, true);

        }

        void check_whether_a_missile_is_launched()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.ControlKey) >= 0)
                return;

            launch_missile(m_plane_object, true);

        }


        void fire_a_bomb(uint parent_object, double speed, bool is_my_bomb)
        {
            DateTime now = DateTime.Now;
            TimeSpan ts = now - m_last_time_a_weapon_was_fired;
            if (ts.TotalMilliseconds < 500)
                return;

            m_last_time_a_weapon_was_fired = now;


            uint bomb_master_copy = m_levels[m_game_level].bomb.object_handle;

            if (bomb_master_copy == 0)
                return; //safty

            uint bomb = object_create_copy(bomb_master_copy, _3D.NO);

            if (bomb == 0)
                return; //safty

            if (is_my_bomb)
                _3D.STATE_object_set_control_type_number(bomb, (int)ObjectOwner.Me);
            else
                _3D.STATE_object_set_control_type_number(bomb, (int)ObjectOwner.Computer);

            _3D.STATE_object_cancel_collision_test_for_chase_physics(bomb);

            //in network gme all units must be relative to time
            //_3D.STATE_object_set_speed_units(_3D.UNITS_PER_RENDER);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(parent_object, location);

            location[2] -= 5000;
            _3D.STATE_object_set_location1(bomb, location);

            _3D.STATE_object_set_chase_type(bomb, _3D.CHASE_PHYSICS);
            double[] direction = new double[3];
            _3D.STATE_object_get_direction1(parent_object, direction);

            _3D.STATE_object_set_speed(bomb, direction);
            _3D.STATE_object_set_absolute_speed(bomb, speed);

            double[] force = { 0, 0, -200 };
            _3D.STATE_object_set_force(bomb, force);

            _3D.STATE_object_set_friction(bomb, 0.001);

            //not relevant when collision detection is canceled
            //  _3D.STATE_object_set_elasticity(bomb, 0);

            //int test=_3D.STATE_object_get_object_type_number(bomb);


            _3D.STATE_object_enable(bomb);

            m_bombs.Add(bomb);


        }

        DateTime m_last_time_a_weapon_was_fired = new DateTime(1980, 1, 1);

        double m_player_delay_between_missiles = 0.5;

        uint launch_missile(uint parent_object, bool is_my_missile)
        {

            if (is_my_missile)
            {
                DateTime now = DateTime.Now;
                TimeSpan ts = now - m_last_time_a_weapon_was_fired;
                if (ts.TotalSeconds < m_player_delay_between_missiles)
                    return (0);

                m_last_time_a_weapon_was_fired = now;
            }

            uint missile_master_copy;
            if(is_my_missile)
                missile_master_copy = m_levels[m_game_level].player_missile.object_handle;
            else
                missile_master_copy = m_levels[m_game_level].enemy_missile_obj;


            if (missile_master_copy == 0)
                return (0); //safty

            uint missile = object_create_copy(missile_master_copy, _3D.NO);

            if (missile == 0)
                return (0); //safty

            if (is_my_missile)
                _3D.STATE_object_set_control_type_number(missile, (int)ObjectOwner.Me);
            else
                _3D.STATE_object_set_control_type_number(missile, (int)ObjectOwner.Computer);


            _3D.STATE_object_cancel_collision_test_for_chase_physics(missile);

            //in network gme all units must be relative to time (not per render)
            //_3D.STATE_object_set_speed_units(_3D.UNITS_PER_RENDER);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(parent_object, location);

            location[2] -= 3000;
            _3D.STATE_object_set_location1(missile, location);

            _3D.STATE_object_set_chase_type(missile, _3D.CHASE_PHYSICS);
            double[] direction = new double[3];
            if (is_my_missile)
                _3D.STATE_object_get_direction1(parent_object, direction);
            else
                direction=get_enemy_shooting_direction(parent_object, m_current_level.enemy_shooting_style);

            _3D.STATE_object_set_direction1(missile, direction);

            _3D.STATE_object_set_speed(missile, direction);
            _3D.STATE_object_set_absolute_speed(missile, m_missile_speed);

            _3D.STATE_object_set_physics_rotation(missile, _3D.OBJECT_SPACE, 360, 0, 0);

            //double[] force = { 0, 0, -200 };
            //_3D.STATE_object_set_force(missile, force);

            //_3D.STATE_object_set_friction(missile, 0.001);

            //not relevant when collision detection is canceled
            //_3D.STATE_object_set_elasticity(missile, 0);

            _3D.STATE_object_set_object_type_number(missile, (int)ObjectType.Missile);


            _3D.STATE_object_enable(missile);

            if (is_my_missile)
            {
                _3D.STATE_sound_stop(m_player_missile_sound);
                _3D.STATE_sound_play(m_player_missile_sound, _3D.SOUND_NO_LOOP);
                m_bombs.Add(missile);
            }

            return (missile);

        }



        void turn_plane_according_to_keyboard_input(uint plane_object)
        {
            bool arrow_key_was_pressed = false;

            //note that rotation around our airplane Y axis increases the tilt angle (how much the nose is higher or lower than the tail)
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
            {
                _3D.STATE_object_rotate_y(plane_object, 2, _3D.OBJECT_SPACE);
                arrow_key_was_pressed = true;
            }
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
            {
                _3D.STATE_object_rotate_y(plane_object, -2, _3D.OBJECT_SPACE);
                arrow_key_was_pressed = true;
            }

            //note tat rotation around the airplane X axis increases the bank angle (when one wing is higher and the other wind is lower)
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0)
            {
                _3D.STATE_object_rotate_x(plane_object, 2, _3D.OBJECT_SPACE);
                arrow_key_was_pressed = true;
            }
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0)
            {
                _3D.STATE_object_rotate_x(plane_object, -2, _3D.OBJECT_SPACE);
                arrow_key_was_pressed = true;
            }

            if (arrow_key_was_pressed)
            {
                ActiveControl = pictureBox_rendering_window;
            }

            double bank = _3D.STATE_object_get_bank(plane_object);

            //Make the turn proportional to the bank angle. The more one wing is higher and the other one is lower the turn
            //will be bigger
            _3D.STATE_object_rotate_z(plane_object, -bank * 0.04, _3D.OBJECT_SPACE);

            //decrease the bank == try to straighten the plane
            bank *= 0.97;
            _3D.STATE_object_set_bank(plane_object, bank);

            //try to reduce the tilt and thus straighten the plane (so that the nose is at the same hight as the tail)
            double tilt = _3D.STATE_object_get_tilt(plane_object);
            tilt *= 0.95;
            _3D.STATE_object_set_tilt(plane_object, tilt);

        }

        double change_speed_according_to_keyboard_input(double speed, uint font_handle)
        {
            double initial_speed = speed;

            if (_3D.STATE_utilities_get_keyboard_key_state((int)' ') < 0)
                speed += m_plane_max_speed/200;

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'B') < 0)
                speed -= m_plane_max_speed / 200;

            if (speed > m_plane_max_speed)
                speed = m_plane_max_speed;

            if (speed < m_plane_min_speed)
                speed = m_plane_min_speed;


            if (initial_speed == speed)
                return (speed);

            //Write speed to screen
            string speed_str = "Speed: " + ((int)speed).ToString();
            _3D.STATE_font_write_text(font_handle, speed_str, 0, 25, 255, 255, 255, 120, _3D.FONT_WRITE_DEFAULT, 100);


            return (speed);
        }



        bool is_point_inside_the_world(double[] location)
        {
            for (int i = 0; i < 3; i++)
            { //note that <2 and not <3 because we don't care about getting out of the world on the X axis
                if ((location[i] > m_world_bounding_box[1, i] * 0.9) || (location[i] < m_world_bounding_box[0, i] * 0.9))
                    return (false);
            }
            return (true);


        }


        void check_that_the_plane_doesnt_go_out_of_the_terrain()
        {
            double[] location = new double[3];
            _3D.STATE_object_get_location1(m_plane_object, location);
            if (is_point_inside_the_world(location))
                return;

            _3D.STATE_object_set_location(m_plane_object, 0, 0, 5000);
        }


        void make_sure_the_plane_doesnt_go_below_the_ground(uint plane_object)
        {


            //Make sure the plane doesn't get into the ground
            double max_height, min_height;

            _3D.STATE_object_get_bounding_box_height_above_ground(plane_object, _3D.YES, out min_height, out max_height);

            if (min_height < m_plane_min_height)
            {
                _3D.STATE_object_move(plane_object, _3D.WORLD_SPACE, 0, 0, m_plane_min_height - min_height);

            }

        }




        void check_level_time_limit()
        {
            if (m_game_status != GameStatus.Playing)
                return;


            if (m_current_level.level_time_limit_in_minutes <= 0)
                return;

            string time_left_str;
            TimeSpan time_left;
            bool did_time_passed = m_current_level.is_level_time_over(out time_left_str, out time_left);

            wrtie_time_left_to_screen(time_left_str, time_left);

            if (did_time_passed == false)
                return;

            m_game_status = GameStatus.GameOver;
            show_game_gui_controls(false);            

            //appears 3 times: when time is finished, when get killed when all level dinished
            //should do one function that is called for each gameover ...

            _3D.STATE_object_disable(m_plane_object);


        }



        void wrtie_time_left_to_screen(string time_left_str, TimeSpan time_left)
        {
            if (time_left.TotalSeconds > 10)
            {
                _3D.STATE_font_write_text(m_ariel18, time_left_str, 90, 0, 255, 255, 0, 0,  _3D.FONT_WRITE_CENTERED_X, 1);
                return;
            }

            byte transparent = (byte)(255 - time_left.TotalSeconds * 15); //at the beginning we have 10 seconds so transparency will be 155

            _3D.STATE_font_write_text(m_ariel_title, time_left_str, 50, 50, transparent, 255, 0, 0, _3D.FONT_WRITE_CENTERED_X | _3D.FONT_WRITE_CENTERED_Y, 1);

            if (_3D.STATE_sound_is_sound_playing(m_beep_sound) == _3D.YES)
                return;

            if (time_left.TotalSeconds < 0.3)
                return; //dont beep when time is almost finished


            if (time_left.Milliseconds < 100) //beep on the beginning of every second
                _3D.STATE_sound_play(m_beep_sound, _3D.SOUND_NO_LOOP);

        }


        //return the selected level
        int show_main_menu_dialog()
        {
            MainMenu mm = new MainMenu();

            for (int i = 0; i < m_levels.Length; i++)
            {
                mm.m_level_names.Add((i+1).ToString() + ") " + m_levels[i].level_name);
                
            }

            mm.fill_combo_box();

            mm.ShowDialog();

            checkBox_practice_mode.Checked = mm.m_is_practice;

            m_game_status = GameStatus.Level_Is_Finished;

            return (mm.m_selected_level);

        }



        //if rendering is too slow it will shut down lighting and shadow genereation
        //in order to speed up rendering
        void control_rendering_speed()
        {
            double t = _3D.STATE_engine_get_computer_speed_factor();
            if (t == 0)
                return; //STATE_engine_render() wasnt called even once. protect against division by zero

            uint fps = (uint)Math.Round((double)(10 / t));

            
            //write "frames per second" value to screen
            //_3D.STATE_font_write_text(m_ariel18, fps.ToString(), 0, 65, 255, 55, 155, 200, _3D.FONT_WRITE_DEFAULT, 100);
            
            if (checkBox_auto_graphics.Checked == false)
                return;

            if ((checkBox_light.Checked == false) && (checkBox_shadows.Checked == false))
                return; //everything is switched of so we have nothing to do here

            if (m_game_status != GameStatus.Playing)
                return;



            if (fps < 10)
            {
                DialogResult dr=MessageBox.Show("To increase graphics speed, we will now cancel shadows and lighting", "Increase speed ?", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                if (dr == DialogResult.OK)
                {
                    checkBox_light.Checked = false;
                    checkBox_shadows.Checked = false;
                }
                else
                    checkBox_auto_graphics.Checked = false; //user selected cancel. so we dont want this popup to come again
            }
            
            

            
        }

        void show_game_gui_controls(bool show)
        {
            checkBox_F16.Visible = show;
            checkBox_engine_sound.Visible = show;
            checkBox_background.Visible = show;
            button_quit_game.Visible = show;
            checkBox_practice_mode.Visible = show;
            button_help.Visible = show;
            //groupBox_shot_down.Visible = show;
            groupBox_graphics.Visible = show;
            label_help_on_keys.Visible = show;
            groupBox_camera_chase_type.Visible = show;
        }

        double m_alpha_effect = 0;

        void flash_the_gameover_text()
        {
            string text = "Game Over";
            if (m_game_was_completed == true)
                text = "Game Completed !";

            //the cos() function is a cyclic function going from 1 to -1 back and forth.
            //note that we use the cos function simply to get a value that starts with 255 then goes down to 0 then go up again to 255 and so on
            //this causes the text to apear and disapear (totally disapear when alpha is 0 and fully seen when alpha is 255)
            _3D.STATE_font_write_text(m_ariel_title, text, 50, 40, (byte)(127 * Math.Cos(m_alpha_effect) + 128), 255, 0, 0, _3D.FONT_WRITE_CENTERED_X, 1);


            _3D.STATE_font_write_text(m_ariel_title, "Press the Continue button", 50, 60, (byte)(127 * Math.Cos(m_alpha_effect + Math.PI / 2) + 128), 255, 0, 0, _3D.FONT_WRITE_CENTERED_X, 1);

            m_alpha_effect += 0.02;

            check_that_the_camera_doesnt_go_under_the_ground();

            button_continue.Visible = true;
            return;

        }


        #region events

        private void radioButton_precise_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;

            if (radioButton_precise.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_PRECISE);
            numericUpDown_chase_distance.Enabled = true;


        }

        private void radioButton_flexible_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;
            if (radioButton_flexible.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_FLEXIBLE);
            numericUpDown_chase_distance.Enabled = true;

        }

        private void radioButton_no_chase_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;

            if (radioButton_no_chase.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.NO_CHASE);
            numericUpDown_chase_distance.Enabled = false;


        }



        private void numericUpDown_chase_distance_ValueChanged(object sender, EventArgs e)
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_set_chase_distance(camera, (double)numericUpDown_chase_distance.Value); //set the camera behind the airplane

        }



        private void pictureBox_rendering_window_Click(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;
        }


        void use_shadows(bool flag)
        {
            int ON_or_OFF = _3D.OFF;
            if (flag)
                ON_or_OFF = _3D.ON;

            
            
            _3D.STATE_object_auto_add_shadows(m_plane_object, ON_or_OFF, 1);

            if (m_enemy_planes.Count > 5)
                return; //using shadows is time consuming so we only use it when we have 5 or less enmies planes
                        //if there are more, we never switch shadows on therefor we never need to disable it

            foreach(uint enemy_plane in m_enemy_planes)
                _3D.STATE_object_auto_add_shadows(enemy_plane, ON_or_OFF, 1);


        }
        
        
        private void checkBox_shadows_CheckedChanged(object sender, EventArgs e)
        {
            use_shadows(checkBox_shadows.Checked);

        }

        private void checkBox_engine_sound_CheckedChanged(object sender, EventArgs e)
        {

            if (checkBox_engine_sound.Checked)
            {
                if (_3D.STATE_sound_is_sound_playing(m_propellor_sound) == _3D.NO)
                    _3D.STATE_sound_play(m_propellor_sound, _3D.SOUND_LOOP_NORMAL);

                return;
            }

            if (_3D.STATE_sound_is_sound_playing(m_propellor_sound) == _3D.YES)
                _3D.STATE_sound_stop(m_propellor_sound);


        }

        private void checkBox_background_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_background.Checked)
            {
                if (_3D.STATE_sound_is_sound_playing(m_background_music) == _3D.NO)
                    _3D.STATE_sound_play(m_background_music, _3D.SOUND_LOOP_NORMAL);

                return;
            }

            if (_3D.STATE_sound_is_sound_playing(m_background_music) == _3D.YES)
                _3D.STATE_sound_stop(m_background_music);


        }

        private void checkBox_F16_CheckedChanged(object sender, EventArgs e)
        {
            uint piper_plane = m_levels[m_game_level].alternative_plane.object_handle;
            uint main_plane = m_levels[m_game_level].player_plane.object_handle;

            if (main_plane == piper_plane)
                return;
            
            uint prev_plane, new_plane;
            if (checkBox_F16.Checked)
            {
                prev_plane = piper_plane;
                new_plane = main_plane;
            }
            else
            {
                prev_plane = main_plane;
                new_plane = piper_plane;
            }

            _3D.STATE_object_enable(new_plane);
            _3D.STATE_object_disable(prev_plane);

            double[] location = new double[3];
            double[] direction = new double[3];

            _3D.STATE_object_get_location1(prev_plane, location);
            _3D.STATE_object_get_direction1(prev_plane, direction);

            _3D.STATE_object_set_location1(new_plane, location);
            _3D.STATE_object_set_direction1(new_plane, direction);

            _3D.STATE_camera_set_object_to_chase(m_camera, new_plane);

            _3D.STATE_sound_attach(m_propellor_sound, new_plane);

            m_plane_object = new_plane;

            //to set the shadow setting for the new used plane
            use_shadows(checkBox_shadows.Checked);

            ActiveControl = pictureBox_rendering_window; //move the focus to the rendering window so clicking on space wont popup the About window again

        }

        private void button_help_Click(object sender, EventArgs e)
        {
            Level level = m_levels[m_game_level];
            
            if (level == null)
                return; //safty

            string specific_level_help = level.level_text.Replace("\n", "\r\n\r\n");
            string general_help = label_help_on_keys.Text;

            string text = specific_level_help + "\r\n\r\n" + general_help;
            //MessageBox.Show(text, "Level Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);
            _3D.STATE_font_write_text(m_ariel18, text, 25, 10, 255, 0, 200, 50,  _3D.FONT_WRITE_DEFAULT, 400);



        }

        private void checkBox_light_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_light.Checked)
            {
                if (m_light == 0)
                    create_light();

                _3D.STATE_light2_disable_all_lighting(false);


                _3D.STATE_light2_enable(m_light);
                return;
            }

            //_3D.STATE_light2_disable(m_light);
            _3D.STATE_light2_disable_all_lighting(true);

        }

        private void button_about_Click(object sender, EventArgs e)
        {
            About ab = new About();

            ab.ShowDialog();

            ActiveControl = pictureBox_rendering_window; //move the focus to the rendering window so clicking on space wont popup the About window again

        }




        int m_show_quit_messagebox_only_3_times = 3; //one can use it to get a pause and get away from the enemies ...
        private void button_quit_game_Click(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;
           
            
            if (m_game_status != GameStatus.Playing)
                return;


            if ( (m_show_quit_messagebox_only_3_times > 0) || checkBox_practice_mode.Checked)
            {
                DialogResult dr = MessageBox.Show("Are you sure you want to exit the game ?", "Exit ?", MessageBoxButtons.YesNo);
                if (dr == DialogResult.No)
                {
                    m_show_quit_messagebox_only_3_times--;
                    return;
                }
            }

            m_game_status = GameStatus.Player_Was_Killed;
            kill_player_plane(m_plane_object);

            _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_FLEXIBLE);

        }


        #endregion events


    }

}