﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;




namespace OneAgainst100Planes
{
    public partial class Game : Form
    {
        int m_game_level = 0; //increased by 1 each time the player goes to the next level. The first level is 0
        int m_starting_level; //the initial level that we started (a player doesnt have to start from the beginning each game ....)

        private EventHandler m_idleEventHandler;
        bool m_exiting = false;

        uint m_camera;
        uint m_background_music;
        uint m_propellor_sound;
        uint m_explosion_sound;
        uint m_pain_sound;
        uint m_beep_sound;

        const int m_number_of_simultaneous_sounds = 5; //meaning we will hear the closest 5 enemy planes plus the closest 5 missiles plus the closest 5 tanks

        uint[] m_enemy_tank_sounds;
        uint[] m_enemy_missile_sounds;
        uint m_player_missile_sound;

        uint m_ariel18;
        uint m_ariel_title;



        double m_bomb_killing_zone = 20000;
        double m_missile_killing_zone = 10000;
        double m_missile_speed = 100000;


        uint m_track1;


        double[,] m_world_bounding_box;
        double m_highest_mountain_top;

        double m_fog_visible_distance = 1500000;

        uint m_plane_object;
        double m_plane_max_speed = 50000;
        double m_plane_min_speed = 15000; 
        double m_plane_speed = 30000; //virtual units per second 
                                    //we are working in units per second
                                    //because this is also the units that the physics engine uses (CHASE_PHYSICS).

        double m_camera_min_height = 4000; ////height above the ground
        double m_plane_min_height = 6000; //height above the ground

        GameStatus m_game_status = GameStatus.MainMenu;
        bool m_game_was_completed=false; //When the player has finished all the levels

        // possible scenarios:
        // 1) Playing -> Level_Is_Finished -> Playing ...
        // 2) MainMenu --(click button)--> Level_Is_Finished --(press 'N')-->Playing ->Player_Was_Killed -> GameOver --(press 'S')--> MainMenu ...
        //
        enum GameStatus { Playing, Level_Is_Finished,Player_Was_Killed, GameOver, MainMenu}

        List<uint> m_enemy_planes = new List<uint>();
        List<uint> m_enemy_tanks = new List<uint>();

                           
        Level m_current_level;

        uint m_light;

        static public int m_total_number_of_object_created = 0; //the number of 3dstate object we create. for debugging. to see that the number of object is not increasing to infinity ...
        static public Dictionary<uint, string> m_allocated_3d_objects_dict=new Dictionary<uint,string>(100); //for debugging

        public Game()
        {
            InitializeComponent();
        }

        private void Airplane2_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;

        }

 


        //the form loading event
        private void Airplane2_Load(object sender, EventArgs e)
        {
            
            //Play some background music while the wprld is loaded.
            m_background_music = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Elgar.mp3", 0, _3D.SOUND_DISTANCE_DEFAULT);
            _3D.STATE_sound_set_volume(m_background_music, 800);

            _3D.STATE_sound_play(m_background_music, _3D.SOUND_LOOP_NORMAL);


            //load_world            
            load_world("..\\..\\..\\..\\worlds\\terrain2\\terrain2.3dstate");


            m_starting_level = show_main_menu_dialog(); //must be at the beginning here since we might change the current selected level.
            m_game_level = m_starting_level;
            m_current_level = m_levels[m_game_level];

            m_current_level.load_level_models();
            m_plane_object = m_current_level.player_plane.object_handle;
            
            create_enemy_objects();
            
            
            load_sounds();

            m_ariel18 = _3D.STATE_font_create("Ariel", 18, _3D.FONT_CREATE_DEFAULT);
            m_ariel_title = _3D.STATE_font_create("Ariel", 36, _3D.FONT_CREATE_BOLD);

            make_the_camera_follow_the_airplane_automatically();

            _3D.STATE_engine_set_fog(_3D.ON, 220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance, _3D.FOG_START_DISTANCE_DEFAULT, 0);

            //makes collision detection calculations faster. for missile hitting targets, we do it ourselves ...
            //so we just need the engine collision detection to test against the terrain
            _3D.STATE_engine_ignore_dynamic_objects_for_collision_detection(_3D.YES);

            //do the first render. also loads the textures
            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            //can only be called after at least one render
            _3D.STATE_3D_card_set_texture_tiling_mode(_3D.TEXTURE_TILING_WRAP);


            checkBox_background.Checked = false;
            _3D.STATE_sound_stop(m_background_music);

            string[] texts = m_current_level.level_text.Split('\n');
            m_3d_text = new Text3D(texts, m_ariel_title);
            

            //add our OnIdle function to the application on idle event
            //see more details at the Onidle function in this page
            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;



            ActiveControl = pictureBox_rendering_window;

        }


        bool load_world(string world_file_name)
        {

            int was_loaded_ok = _3D.STATE_engine_load_world(world_file_name, "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }

            //Get the world dimesions so that we can watch later
            //that the plane doesn't get out of the world ...
            m_world_bounding_box = new double[2, 3];
            _3D.STATE_group_get_bounding_box(0, m_world_bounding_box);
            m_highest_mountain_top = m_world_bounding_box[1, 2];

            double x_width = m_world_bounding_box[1, 0] - m_world_bounding_box[0, 0];
            double y_width = m_world_bounding_box[1, 1] - m_world_bounding_box[0, 1];
            double max_side = Math.Max(x_width, y_width);

            //add some height to the bounding box so that planes can fly above the mountain hight ...
            double z_width = m_world_bounding_box[1, 2] - m_world_bounding_box[0, 2];
            m_world_bounding_box[1, 2] += z_width * 20 + 5000;


            m_camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_distance_from_eye(m_camera, max_side / 4000);


            //Put some ground details texture
            uint ground_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\terrain2\\Bitmaps\\second_bitmap.bmp", -1);
            if (ground_bmp != 0)
            {
                //Try cancel this line or putting a different number than 100 and see what it does
                _3D.STATE_group_wrap_a_bitmap(0, ground_bmp, 35, 35, 1);
            }



            add_some_light();

            return (true);


        }

        //called when we move to the next level
        void start_game_level(int level_number)
        {
            m_game_was_completed = false;
            //disable previous level objects
            m_current_level.enemy_plane.stop_sounds();

            if (m_current_level.enemy_plane.missile_index == ModelIndex.None)
            {
                if (m_current_level.enemy_missile_obj != 0)
                {
                    delete_object(m_current_level.enemy_missile_obj);
                    m_current_level.enemy_missile_obj = 0;
                }
            }


            m_game_level = level_number;

            m_current_level = m_levels[m_game_level];

            delete_enemies_and_bombs();
            delete_all_dead_enemies();

            m_current_level.load_level_models();
            m_plane_object = m_current_level.player_plane.object_handle;

            if (checkBox_F16.Checked)
                m_current_level.player_plane.enable(true);
            else
                m_current_level.alternative_plane.enable(true);


            //recreate all the enemies
            create_enemy_objects();

            set_enemy_attack_frequency();


            use_shadows(checkBox_shadows.Checked);

            make_the_camera_follow_the_airplane_automatically();

            textBox_plane_hits.Text = "0";
            textBox_tank_hits.Text = "0";

            m_game_status = GameStatus.Playing;
            m_current_level.level_start_time = DateTime.Now;
        }



        void manage_player_missiles()
        {
            if (m_game_status == GameStatus.Playing)
            {
                check_whether_a_bomb_is_fired();
                check_whether_a_missile_is_launched();
            }

            check_whether_a_bomb_hit_enemy();
            check_whether_a_bomb_hit_terrain(m_bombs);
        }

        //----------------------------------------------------
        // The OnIdle is the main function of the game !!!   
        // Here we manage everything !                       
        //----------------------------------------------------
        //
        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface.
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;


            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            
            _3D.STATE_3D_card_set_texture_tiling_mode(_3D.TEXTURE_TILING_WRAP);

            handle_game_status();

            check_level_time_limit();

            move_plane();

            manage_player_missiles();

            manage_enemy_missiles();

            deal_with_killed__enemy_airplanes_reaching_the_ground();

            manage_enemy_attacks(m_enemy_planes, m_current_level.enemy_AI);
            make_sure_enemy_planes_dont_go_under_the_ground(m_enemy_planes);

            control_rendering_speed();

            set_sound_to_near_objects(m_enemy_planes, m_current_level.enemy_plane.model_sound_handles, ObjectType.Plane);
            set_sound_to_near_objects(m_enemy_tanks, m_enemy_tank_sounds, ObjectType.Tank);

            //debug_allocated_objects();            

            make_sure_we_get_more_OnIdle_events();
        }

        void debug_allocated_objects()
        {
            string debug_str = "objects: " + m_total_number_of_object_created.ToString() + " player bombs=" + m_bombs.Count.ToString() + " Enemy bombs=" + m_enemy_missiles.Count.ToString() + " killed=" + m_killed_enemies.Count.ToString(); 
            //_3D.STATE_font_write_text(m_ariel18, debug_str, 0, 35, 255, 255, 0, 120, _3D.FONT_WRITE_DEFAULT, 100);

            string debug_str2 = "enemy planes=" + m_enemy_planes.Count.ToString() + " enemy tanks=" + m_enemy_tanks.Count.ToString();
            //_3D.STATE_font_write_text(m_ariel18, debug_str2, 0, 45, 255, 255, 0, 120, _3D.FONT_WRITE_DEFAULT, 100);

            int master_objects = 0;
            foreach (Model m in AllModels.models)
                if (m.object_handle != 0)
                    master_objects++;


            int total_managed = m_bombs.Count + m_enemy_missiles.Count + m_enemy_planes.Count + m_enemy_tanks.Count + m_killed_enemies.Count + master_objects;
            

            
            int diff = m_total_number_of_object_created - total_managed;

            string debug_str3 = "total managed=" + total_managed.ToString() + "  diff=" + diff.ToString();
            //_3D.STATE_font_write_text(m_ariel18, debug_str3, 0, 55, 255, 255, 0, 120, _3D.FONT_WRITE_DEFAULT, 100);
            _3D.STATE_font_write_text(m_ariel18, "  diff=" + diff.ToString(), 0, 55, 255, 255, 0, 120, _3D.FONT_WRITE_DEFAULT, 100);


        }

        void handle_game_status()
        {
            if (m_game_status == GameStatus.GameOver)
            {
                flash_the_gameover_text();
                return;
            }


            if (m_game_status == GameStatus.MainMenu)
            {
                //wait till the button is clicked on the MainMenu Form
                return;
            }


            if (m_game_status == GameStatus.Player_Was_Killed)
            {
                check_that_killed_player_plane_hit_the_ground();
                return;
            }

            if (m_game_status == GameStatus.Level_Is_Finished)
            {
                //we want to rotate in equal speed on all computer no matter the rendering speed
                double factor = _3D.STATE_engine_get_computer_speed_factor();
                        // note that:    time_of_average_render = factor / 10;
                        //note that:     frames_per_second = 10 / factor;
                
                
                //_3D.STATE_font_write_text(m_ariel18, "  factor=" + factor.ToString(), 20, 65, 255, 55, 200, 120, _3D.FONT_WRITE_DEFAULT, 100);
                m_3d_text.turn_text(1.8*factor); // if we want the whole text to repeat itself (make a whole circle ) every 20 seconds
                                                 //then we need to rotate at 360/20 degrees per second == 18 degrees per second.
                                                 // so we need to rotate at 18/fps degrees each render (fps is the frames per second then )
                                                 // factor/10 is the time of each render. so 10/factor is the frames per second number
                                                 // 18/fps == 18*factor/10 == 1.8*factor

                                                    //so if we have 40 frames per second we will get factor=0.25 (1/40 *10) and 1.6*0.25=0.4
                button_continue.Visible = true;


                return;
            }


        }

        private void button_continue_Click(object sender, EventArgs e)
        {
            button_continue.Visible = false;

            switch (m_game_status)
            {
                case GameStatus.Level_Is_Finished:
                    m_3d_text.clear();

                    //move to the next level
                    start_game_level(m_game_level);
                    break;

                case GameStatus.GameOver:
                    restart_game();
                    break;



            }
        }


        DateTime m_last_time_we_moved_the_plane = new DateTime(1980, 1, 1);

        void move_plane()
        {

            check_that_the_plane_doesnt_go_out_of_the_terrain();
            check_that_the_camera_doesnt_go_under_the_ground();

            
            if (m_game_status != GameStatus.Playing)
                return;

            make_sure_the_plane_doesnt_go_below_the_ground(m_plane_object);

            
            //The speed is in virtual units per hundredth of a second.
            DateTime now = DateTime.Now;
            TimeSpan delta_t = now - m_last_time_we_moved_the_plane;
            double delta_x = m_plane_speed * delta_t.TotalSeconds;
            m_last_time_we_moved_the_plane = now;

            _3D.STATE_object_move(m_plane_object, _3D.OBJECT_SPACE, -delta_x, 0, 0); //move the plane forwad. This is towards the plane negative X axis

            turn_plane_according_to_keyboard_input(m_plane_object);

            m_plane_speed = change_speed_according_to_keyboard_input(m_plane_speed, m_ariel18);


        }


        void make_sure_we_get_more_OnIdle_events()
        {
            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...
        }




    }
}
