﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;
//using Math3D;


namespace OneAgainst100Planes
{
    public partial class Game : Form
    {

        void load_sounds()
        {

            m_explosion_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Explosion1.wav", m_plane_object, 3000000);
            _3D.STATE_sound_set_volume(m_explosion_sound, 500);


            m_enemy_tank_sounds = new uint[m_number_of_simultaneous_sounds];
            for (int i = 0; i < m_enemy_tank_sounds.Length; i++)
            {
                m_enemy_tank_sounds[i] = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\tank.wav", m_plane_object, 600000);
                _3D.STATE_sound_set_volume(m_enemy_tank_sounds[i], 700);

            }


            m_enemy_missile_sounds = new uint[m_number_of_simultaneous_sounds];
            for (int i = 0; i < m_enemy_missile_sounds.Length; i++)
            {
                m_enemy_missile_sounds[i] = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Missile.wav", m_plane_object, 600000);
                _3D.STATE_sound_set_volume(m_enemy_missile_sounds[i], 1000);

            }

            m_player_missile_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Missile.wav", 0, 300000);
            _3D.STATE_sound_set_volume(m_player_missile_sound, 1000);



            m_pain_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\pain2.wav", 0, 0);
            _3D.STATE_sound_set_volume(m_pain_sound, 600);

            m_beep_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\beep.wav", 0, 0);
            _3D.STATE_sound_set_volume(m_beep_sound, 600);




            //Set the sound of the wind/plane
            m_propellor_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\propeller_sound.wav", m_plane_object, 600000);
            _3D.STATE_sound_set_volume(m_propellor_sound, 700);

            if (checkBox_engine_sound.Checked)
                _3D.STATE_sound_play(m_propellor_sound, _3D.SOUND_LOOP_NORMAL);

        }

        void play_sound(uint sound, uint object_making_sound)
        {
            //the sound engine automatically calculates the sound volume according to the distance of the object from the camera)
            _3D.STATE_sound_attach(sound, object_making_sound);
            _3D.STATE_sound_play(sound, _3D.SOUND_NO_LOOP);
        }


        //we have a fix number of sounds which we assign to objects that are close to the camera (the point of view)
        //for example this function might get a list of missiles objects and assign a list of missile sounds (the same sound several times)
        //the function will assign the sounds to the missiles that are closest to the camera.
        //Note that there might be hundreds of missiles flying, if we assign each of them with a sound (the same wav file)
        //it will add a lot of computation load to the sound engine since it will have to mix hundreds of sound together
        //All the missiles use the same wave file but each missile has a different playing position (start of the file, middle etc)
        //each has a different volume that is derived from the distance from the camera.
        //since it is a 3D sound, each missile has a different contribution to the left and right speakers depending on
        //the missile location in the world.
        //in other words: computing the correct accumulated sound (done automatically by the sound engine) is not an easy task
        //and there for we use m_number_of_simultaneous_sounds to limit the number of simultaneously played sounds
        void set_sound_to_near_objects(List<uint> objects, uint[] sounds, ObjectType selected_type)
        {
            //return a list of the nearest objects to the point of view (the camera that is used for rendering)
            uint[] near_objects = get_nearest_objects(objects, selected_type, m_number_of_simultaneous_sounds);

            if (near_objects == null)
                return;

            List<uint> sounds_already_assigned_to_near_objects = new List<uint>(near_objects.Length); //these are sounds we cant use since they are already assigned to a close object
            foreach (uint obj in near_objects)
            {
                uint sound = _3D.STATE_sound_get_handle_using_entity(obj);
                if (sound != 0)
                    sounds_already_assigned_to_near_objects.Add(sound);
            }


            foreach (uint obj in near_objects)
            {
                uint sound = _3D.STATE_sound_get_handle_using_entity(obj);
                if (sound != 0)
                {
                    if (_3D.STATE_sound_is_sound_playing(sound) == _3D.NO)
                        _3D.STATE_sound_play(sound, _3D.SOUND_LOOP_NORMAL);

                    continue;
                }

                //now this object has no sound so we need to get him a sound from those who are not used by the other near objects
                bool found_available_sound = false;
                foreach (uint s in sounds)
                {
                    if (sounds_already_assigned_to_near_objects.Contains(s) == false)
                    {
                        _3D.STATE_sound_attach(s, obj);
                        _3D.STATE_sound_stop(s);
                        _3D.STATE_sound_play(s, _3D.SOUND_LOOP_NORMAL);
                        sounds_already_assigned_to_near_objects.Add(s);
                        found_available_sound = true;
                        break;
                    }
                }

                if (found_available_sound == false)
                {
                    //MessageBox.Show("Internal error. Failed finding free sound");
                }

            }


        }


        //return a list of the nearest objects to the point of view (the camera that is used for rendering)
        uint[] get_nearest_objects(List<uint> objects, ObjectType selected_type, int max_number_of_near_objects)
        {
            List<object_distance> obj_list = new List<object_distance>(objects.Count);

            double[] camera_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, camera_location);
            //_3D.STATE_object_get_location1(m_plane_object, camera_location);

            foreach (uint obj in objects)
            {
                ObjectType obj_type = (ObjectType)_3D.STATE_object_get_object_type_number(obj);

                if (obj_type != selected_type)
                    continue;


                double[] obj_location = new double[3];
                _3D.STATE_object_get_location1(obj, obj_location);

                double distance = _3D.STATE_math_get_square_distance(camera_location, obj_location);

                object_distance obj_dist = new object_distance();
                obj_dist.obj = obj;
                obj_dist.distance = distance;

                obj_list.Add(obj_dist);
            }

            if (obj_list.Count <= 0)
                return (null);


            int array_size = max_number_of_near_objects;
            if (obj_list.Count < array_size)
                array_size = obj_list.Count;

            uint[] near_objects = new uint[array_size];

            for (int i = 0; i < array_size; i++)
            {
                object_distance od = find_object_with_smallest_distance(obj_list);
                near_objects[i] = od.obj;

                obj_list.Remove(od);

            }

            return (near_objects);
        }

        object_distance find_object_with_smallest_distance(List<object_distance> obj_list)
        {
            double min_distance = double.MaxValue;
            object_distance min_od = null;

            foreach (object_distance od in obj_list)
            {
                if (od.distance < min_distance)
                {
                    min_distance = od.distance;
                    min_od = od;
                }
            }

            return (min_od);
        }


    
    }
}