﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;

namespace Alpha_Transparency
{
    public partial class AlphaTransparency : Form
    {
        uint m_room;
        uint m_logo_bmp;
        uint m_camera;
        Timer m_timer;
        
        bool m_exiting = false;
       
        
        public AlphaTransparency()
        {
            InitializeComponent();
        }

        private void AlphaTransparency_Load(object sender, EventArgs e)
        {
    	
	        //Write error messages directly to the rendered screen (and not just to 3dstate.log)
	        //Remove this function when shipping your release version ...
	        _3D.STATE_engine_write_error_messages_to_the_screen(_3D.ON);

	        //Set a 32 bitmap format that with 8 bits alpha channel.
	        //All bitmaps loaded into the engine will be converted to this format
	        _3D.STATE_bitmap_set_default_format(_3D.BITMAP_FORMAT_A8R8G8B8);

	        uint logo=_3D.STATE_object_create_from_file("..\\..\\..\\..\\Models\\logo\\3dstate_logo.wld");

	        //The x file format is Microsoft Direct3D file format
            m_room=_3D.STATE_object_create_from_file("..\\..\\..\\..\\Models\\room\\lobby_skybox.x");

	        if( (logo==0) || (m_room==0) ) {
		        MessageBox.Show("Couldn't find files (3dstate_logo.wld or lobby_skybox.x\n", "Error");
		        return;
	        }

	        //Make the m_room bigger
	        double[] scale_room={10,10,10};
	        _3D.STATE_object_set_scale(m_room, scale_room);
    	
	        // Hint the engine that all the world is inside the m_room object
	        // See 3dstate.h for more details
	        _3D.STATE_engine_using_skybox(m_room);


            m_camera = _3D.STATE_camera_get_default_camera();
    	
	        //Since it is a very small world we should be able to get close to objects
	        //Every thing that is closer to the camera then the given number (0.1)
	        //will be cut out of the scene. See 3dstate.h for more details
	        _3D.STATE_camera_set_distance_from_eye(m_camera, 0.1); 

	        uint logo_polygon=_3D.STATE_polygon_get_handle_using_name("logo");
	        if(logo_polygon==0)
		        return; //safty

	        _3D.STATE_polygon_set_translucent(logo_polygon, _3D.ALPHA_FILTER);
	        _3D.STATE_polygon_use_mipmap(logo_polygon, _3D.NO); //_3D.STATE_bitmap_set_pixel() to understand
								          //Why it is better to cancel mipmaps to polygon which we
								          //change their textures


	        m_logo_bmp=_3D.STATE_bitmap_get_handle("3dstate_logo", -1);
	        if(m_logo_bmp==0)
		        return; //safty

            MessageBox.Show("To draw with the mouse, click and drag the mouse over the 3DSTATE logo.\n\nThis demo shows:\n\n1. Real-time modification of texture pixels\n\n2. Using the alpha channel to control transparency", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);


            m_timer = new Timer();
            m_timer.Interval = 10;
            m_timer.Tick += new EventHandler(timer_Tick);
            m_timer.Start();
            

        }


        private void AlphaTransparency_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
        }

        //set the color of the pixel that is at the cursor position to white
        void put_a_white_pixel_on_the_texture()
        {
            //convert te Cursor.Position the coordinates relative to te pictureBox where we render
            Point pt = pictureBox_rendering_window.PointToClient(Cursor.Position);

            double[] XYZ = new double[3];
            uint obj, poly;
            int rc = _3D.STATE_engine_2D_point_to_3D(pt.X, pt.Y, XYZ, out obj, out poly);
            if ((rc != _3D.OK) || (poly == 0))
                return;

            int x, y;
            _3D.STATE_polygon_get_bitmap_pixel_xy(poly, XYZ, 0, out x, out y);
            byte[] rgba = { 255, 255, 255, 255 };
            _3D.STATE_bitmap_set_pixel(m_logo_bmp, 0, x, y, rgba);

        }


        void timer_Tick(object sender, EventArgs e)
        {

            if (m_exiting)
                return; //so we wont get here after Form_close event

             IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, m_camera);

		    _3D.STATE_object_rotate_z(m_room, 0.4, _3D.WORLD_SPACE);

		    do_logo_alpha_transparency_effect(m_logo_bmp);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.LButton) < 0)
                put_a_white_pixel_on_the_texture();

        }

        private void button_help_Click(object sender, EventArgs e)
        {
            string str = File.ReadAllText("..\\..\\..\\..\\Docs\\Alpha_transparency_help.txt");

            if (String.IsNullOrEmpty(str))
                str = "couldnt find help file Alpha_transparency_help.txt";

            MessageBox.Show(str, "Some more info", MessageBoxButtons.OK, MessageBoxIcon.Information);

        }



    }
}
