﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using _3dstate;

namespace Tank
{
    public partial class Tank : Form
    {

        double m_angle = 1;

        void show_help_popup()
        {
            MessageBox.Show("Instructions:\n\nPress F1 To get this help popup\n\nUse the Left and Right arrow keys to navigate the tank\r\nUse the Up and Down arrow keys to change the speed\r\nP to pause/stop the tank\r\n\r\nPress the Ctrl key to fire a bomb\r\n\r\nPress F to set more fog and V to decrease fog\r\n\r\n", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        void add_some_light()
        {
            //Add a light source
            double[] location = { 0, 0, 10000 }; //put it up in the sky like the sun
            uint light = _3D.STATE_light_create("light1", location);
            _3D.STATE_light_set_type_of_light(light, _3D.LIGHT_AMBIENT | _3D.LIGHT_DIFFUSE);
            _3D.STATE_light_set_ambient(light, 0.8);
            _3D.STATE_light_set_specular(light, 0);
            _3D.STATE_light_set_diffuse(light, 3);
            _3D.STATE_light_activate(light, _3D.LIGHT_SET);

            location[2] = 10000000; //this height will look better with the tank shadows since is simulate the sun and the sun is very high above ...
            _3D.STATE_light_set_location(light, location);


        }


        void make_the_camera_follow_the_tank_automatically()
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_set_chase_type(camera, _3D.CHASE_PRECISE);
            _3D.STATE_camera_set_object_to_chase(camera, m_tank_object);
            _3D.STATE_camera_set_chase_distance(camera, (double)numericUpDown_distance.Value); //set the camera behind the tank
            double[] offset = new double[3];
            offset[0] = (double)numericUpDown_x_offset.Value;
            offset[1] = (double)numericUpDown_y_offset.Value;
            offset[2] = (double)numericUpDown_z_offset.Value;
            _3D.STATE_camera_set_chase_offset(camera, offset);
            radioButton_precise.Checked = true;

            _3D.STATE_camera_set_chase_softness(m_camera, (double)numericUpDown_chase_softness.Value);
        }



        void check_that_the_tank_doesnt_go_out_of_the_terrain()
        {
            double[] location = new double[3];
            _3D.STATE_object_get_location1(m_tank_object, location);
            for (int i = 0; i < 2; i++)
            { //note that <2 and not <3 because we don't tanke about getting out of the world on the X axis
                if ((location[i] > m_world_bounding_box[1, i] * 0.9) || (location[i] < m_world_bounding_box[0, i] * 0.9))
                {
                    _3D.STATE_object_set_location(m_tank_object, 0, 0, 5000);
                }
            }

        }

        void check_that_the_camera_doesnt_go_under_the_ground()
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            double height;
            uint ground_poly = _3D.STATE_camera_get_height_above_ground(camera, 0, out height);
            if (ground_poly != 0)
            {
                if (height < m_camera_min_height)
                {
                    _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, m_camera_min_height - height);
                }

                return;
            }

            _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, 1e10);
            ground_poly = _3D.STATE_camera_get_height_above_ground(camera, 0, out height);
            if(ground_poly!=0)
                _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, m_camera_min_height - height);


        }

        void change_settings_according_to_keyboard_input()
        {
            //Show the help popup
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.F1) < 0)
                show_help_popup();

            uint camera = _3D.STATE_camera_get_default_camera();

            change_fog_settings_according_to_keyboard_input();

        }


        void turn_tank_according_to_keyboard_input(uint tank_object)
        {

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0) _3D.STATE_object_rotate_z(tank_object, m_angle, _3D.OBJECT_SPACE);
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0) _3D.STATE_object_rotate_z(tank_object, -m_angle, _3D.OBJECT_SPACE);


        }

        double m_speed_before_pause = 300;
        
        double change_speed_according_to_keyboard_input(double speed, uint font_handle)
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
            {
                if ((speed > 30) || (speed < -30))
                    speed += 5;
                else
                    speed += 1;

                ActiveControl = pictureBox_rendering_window; //so that the numericupdown control wont get the input

            }

            if ( (_3D.STATE_utilities_get_keyboard_key_state((int)'P') & 1)>0)
            {
                if (speed == 0)
                    speed = m_speed_before_pause;
                else
                    speed = 0;

                return (speed);
            }


            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
            {
                if ((speed > 30) || (speed < -30))
                    speed -= 5;
                else
                    speed -= 1;

                ActiveControl = pictureBox_rendering_window; //so that the numericupdown control wont get the input
            }

            if (speed > m_tank_max_speed)
                speed = m_tank_max_speed;

            if (speed < m_tank_min_speed)
                speed = m_tank_min_speed;

            //Write speed to screen
            string speed_str = "Speed: " + ((int)speed).ToString();
            _3D.STATE_font_write_text(font_handle, speed_str, 0, 25, 255, 0, 255, 120, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);


            return (speed);
        }








        //decrease or increase the fog intensity according to the keyboard input
        void change_fog_settings_according_to_keyboard_input()
        {
            double factor = 1;
            if (_3D.STATE_utilities_get_keyboard_key_state((int)'F') < 0)
            {
                factor = 0.95;
            }
            if (_3D.STATE_utilities_get_keyboard_key_state((int)'V') < 0)
            {
                factor = 1.05;
            }

            if (factor == 1)
                return; //no change is needed. Neither of the keys is pressed

            m_fog_visible_distance *= factor;


            if (m_fog_visible_distance < m_fog_min_visible_distance)
                m_fog_visible_distance = m_fog_min_visible_distance;


            if (m_fog_visible_distance > m_fog_max_visible_distance)
            {
                m_fog_visible_distance = m_fog_max_visible_distance;
                _3D.STATE_engine_set_fog(_3D.OFF, 220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance, _3D.FOG_START_DISTANCE_DEFAULT, 0);
                _3D.STATE_font_write_text(m_ariel18, "Fog: OFF", 0, 50, 255, 0, 0, 100, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
                return;
            }


            _3D.STATE_engine_set_fog(_3D.ON, 220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance, _3D.FOG_START_DISTANCE_DEFAULT, 0);

            string fog_text = "Fog intensity: " + (int)m_fog_visible_distance;
            _3D.STATE_font_write_text(m_ariel18, fog_text, 0, 90, 255, 0, 0, 100, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);

        }


        #region windows GUI events
        private void radioButton_precise_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;

            if (radioButton_precise.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_PRECISE);
            numericUpDown_distance.Enabled = true;


        }

        private void radioButton_flexible_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;
            if (radioButton_flexible.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_FLEXIBLE);
            numericUpDown_distance.Enabled = true;

        }

        private void radioButton_location_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;

            if (radioButton_location.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.CHASE_LOCATION);
            numericUpDown_distance.Enabled = false;

        }

        private void radioButton_no_chase_CheckedChanged(object sender, EventArgs e)
        {
            ActiveControl = pictureBox_rendering_window;

            if (radioButton_no_chase.Checked == false)
                return;

            _3D.STATE_camera_set_chase_type(m_camera, _3D.NO_CHASE);
            numericUpDown_distance.Enabled = false;


        }

        private void numericUpDown_distance_ValueChanged(object sender, EventArgs e)
        {
            _3D.STATE_camera_set_chase_distance(m_camera, (double)numericUpDown_distance.Value);
        }

        private void numericUpDown_x_offset_ValueChanged(object sender, EventArgs e)
        {
            double[] offset = new double[3];
            _3D.STATE_camera_get_chase_offset(m_camera, offset);
            offset[0] = (double)numericUpDown_x_offset.Value;
            _3D.STATE_camera_set_chase_offset(m_camera, offset);
        }

        private void numericUpDown_y_offset_ValueChanged(object sender, EventArgs e)
        {
            double[] offset = new double[3];
            _3D.STATE_camera_get_chase_offset(m_camera, offset);
            offset[1] = (double)numericUpDown_y_offset.Value;
            _3D.STATE_camera_set_chase_offset(m_camera, offset);

        }

        private void numericUpDown_z_offset_ValueChanged(object sender, EventArgs e)
        {
            double[] offset = new double[3];
            _3D.STATE_camera_get_chase_offset(m_camera, offset);
            offset[2] = (double)numericUpDown_z_offset.Value;
            _3D.STATE_camera_set_chase_offset(m_camera, offset);

        }

        private void numericUpDown_chase_softness_ValueChanged(object sender, EventArgs e)
        {
            _3D.STATE_camera_set_chase_softness(m_camera, (double)numericUpDown_chase_softness.Value);

        }

        private void checkBox_background_music_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_background_music.Checked)
            {
                if (_3D.STATE_sound_is_sound_playing(m_background_music) == _3D.NO)
                    _3D.STATE_sound_play(m_background_music, _3D.SOUND_LOOP_NORMAL);

                return;
            }

            if (_3D.STATE_sound_is_sound_playing(m_background_music) == _3D.YES)
                _3D.STATE_sound_stop(m_background_music);


        }

        private void checkBox_tank_engine_sound_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_tank_engine_sound.Checked)
            {
                if (_3D.STATE_sound_is_sound_playing(m_tank_engine_sound) == _3D.NO)
                    _3D.STATE_sound_play(m_tank_engine_sound, _3D.SOUND_LOOP_NORMAL);

                return;
            }

            if (_3D.STATE_sound_is_sound_playing(m_tank_engine_sound) == _3D.YES)
                _3D.STATE_sound_stop(m_tank_engine_sound);

        }


        #endregion 


    }

}