﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using _3dstate;
using Math3D;

namespace Tank
{
    public partial class Tank : Form
    {
        uint m_bomb_object; //the master bomb that is being duplicated
        List<uint> m_bombs = new List<uint>();
        uint m_explosion_object; ////the master copy of the explosion that is being duplicated
        uint m_explosion_sound;
        double m_explosion_sound_radius = 250000;
        int m_min_explosion_sound = 30; //maximun is 1000


        double m_bomb_initial_speed = 5000;


        DateTime m_last_time_a_bomb_was_fired = DateTime.Now;
        
        void check_whether_a_bomb_is_fired()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.ControlKey) >= 0)
                return;

            DateTime now = DateTime.Now;
            TimeSpan ts = now - m_last_time_a_bomb_was_fired;

            if (ts.TotalMilliseconds < 300) //allow only 3 shels/bombs in a second
                return;

            m_last_time_a_bomb_was_fired = DateTime.Now;

            uint bomb = _3D.STATE_object_duplicate(m_bomb_object, _3D.YES);

            _3D.STATE_object_cancel_collision_test_for_chase_physics(bomb);
            _3D.STATE_object_set_speed_units(_3D.UNITS_PER_RENDER);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(m_tank_object, location);

            //location[2] +=m_tank_height/2;

            _3D.STATE_object_set_location1(bomb, location);

            _3D.STATE_object_set_chase_type(bomb, _3D.CHASE_PHYSICS);
            double[] direction = new double[3];
            _3D.STATE_object_get_direction1(m_tank_object, direction);

            _3D.STATE_object_set_speed(bomb, direction);
            _3D.STATE_object_set_absolute_speed(bomb, m_bomb_initial_speed);

            double[] force = { 0, 0, -20 };
            _3D.STATE_object_set_force(bomb, force);

            _3D.STATE_object_set_friction(bomb, 0.001);

            _3D.STATE_object_set_elasticity(bomb, 0);


            _3D.STATE_object_enable(bomb);

            m_bombs.Add(bomb);


        }


        void explode_bomb(uint bomb)
        {
            uint explosion = _3D.STATE_object_duplicate(m_explosion_object, _3D.NO);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(bomb, location);

            _3D.STATE_object_set_location1(explosion, location);

            double length, width, height;
            _3D.STATE_object_get_dimensions(explosion, out length, out width, out height);
            _3D.STATE_object_move(explosion, _3D.WORLD_SPACE, 0, 0, height / 2);


            _3D.STATE_object_delete(bomb);

            _3D.STATE_object_set_event(explosion, 5000, _3D.STATE_DELETE);

            int volume=get_explosion_sound_volume(location);

            _3D.STATE_sound_set_volume(m_explosion_sound, volume);
            
            _3D.STATE_sound_play(m_explosion_sound, _3D.SOUND_NO_LOOP);

        }

        int get_explosion_sound_volume(double[] explosion_location)
        {
            double[] camera_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, camera_location);

            double root_distance = Math.Sqrt(Vector.distance_between_points(explosion_location, camera_location));
            double root_sound_radius = Math.Sqrt(m_explosion_sound_radius);
           
            //a linear equasion which will return 1000 when distance==0 and will return m_min_explosion_sound when distance=m_explosion_sound_radius
            double volume = -1 * (1000 - m_min_explosion_sound) * root_distance / root_sound_radius + 1000;

            if (volume < m_min_explosion_sound)
                volume = m_min_explosion_sound;

            if (volume > 1000)
                volume = 1000;


            int volume_int=(int)volume;
            textBox_last_explosion_volume.Text = volume_int.ToString();
            
            return(volume_int);


        }

        void check_whether_a_bomb_has_to_explode()
        {
            List<uint> bombs_to_delete = new List<uint>();

            foreach (uint bomb in m_bombs)
            {
                double height_above_ground = _3D.STATE_object_get_center_height_above_ground(bomb, _3D.YES);
                if (height_above_ground > 0)
                    continue;

                explode_bomb(bomb);
                bombs_to_delete.Add(bomb);

            }

            foreach (uint bomb in bombs_to_delete)
                m_bombs.Remove(bomb);


        }


        bool load_explosion(string file_name)
        {
            

            m_explosion_object = _3D.STATE_object_create_from_file(file_name);
            if (m_explosion_object == 0)
            {
                MessageBox.Show("Failed loading explosion model.\r\nFile=" + file_name, "Failed loading airplane model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (false); //safty
            }

            double[] scale = { 2000, 2000, 2000 }; //make the plane bigger
            _3D.STATE_object_set_scale(m_explosion_object, scale);

            //The object is turned around so lets turn it 180 degrees
            //This is important so that the _3D.STATE_chase functions work as
            //we expect them to work
            //_3D.STATE_object_rotate_z(m_explosion_object, 180, _3D.WORLD_SPACE);
            //_3D.STATE_object_redefine_axis_system(m_explosion_object);


            _3D.STATE_object_disable(m_explosion_object);
            return (true);



        }


        bool load_bomb(string file_name)
        {
            m_bomb_object = _3D.STATE_object_create_from_file(file_name);
            if (m_bomb_object == 0)
            {
                MessageBox.Show("Failed loading bomb model.\r\nFile=" + file_name, "Failed loading airplane model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (false); //safty
            }

            double[] scale = { 10, 10, 10 }; //make the plane bigger
            _3D.STATE_object_set_scale(m_bomb_object, scale);

            //The object is turned around so lets turn it 180 degrees
            //This is important so that the _3D.STATE_chase functions work as
            //we expect them to work
            //_3D.STATE_object_rotate_z(m_bomb_object, 180, _3D.WORLD_SPACE);
            //_3D.STATE_object_redefine_axis_system(m_bomb_object);


            _3D.STATE_object_disable(m_bomb_object);
            return (true);


        }

        




    }
}