﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using _3dstate;
using Math3D;



namespace TrainYard2
{
    public partial class TrainYard2 : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        double m_walking_step = 2;
        //double m_running_step = 3;
        double m_angles_to_turn = 2;

        uint m_shooting_mark_bmp;
        uint m_shooting_sound;
        uint[] m_step_sounds = new uint[4];
        uint m_fall_sound;
        uint m_hit_wall_sound;
        uint m_pain_sound;

        uint m_fire_effect_polygon_front; //the flashing fire at the edgwe of the weapon
        uint m_fire_effect_polygon_back; //the back side so when we see the face of the model will also see the fire effect
        uint m_fire_animation;
        uint m_master_shred;
        uint m_weapon_edge_point;
        Random m_rnd = new Random();

        uint m_body_obj, m_legs_obj, m_weapon_obj, m_head_obj;
        double m_model_height;

        double m_camera_distance_from_object = 80;

        uint m_camera;
        uint m_temp_camera; //use to test collision since we have STATE_camera_move_with_collision_detection()
                            //but we dont have a similar STATE_object_move_with_collision_detection()
                            //so we will put a camera in the place of the object to test collision

        uint m_light;
        uint m_crosshair_layer;
        string[] m_skin_files = { "..\\..\\..\\..\\models\\human_character\\skins\\skin1.jpg", "..\\..\\..\\..\\models\\human_character\\skins\\skin2.jpg", "..\\..\\..\\..\\models\\human_character\\skins\\skin3.jpg", "..\\..\\..\\..\\models\\human_character\\skins\\skin4.jpg" };

        double m_camera_projection_palne_distance = 0.1;

        public TrainYard2()
        {
            InitializeComponent();
        }

        private void TrainYard2_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;
        }

        private void TrainYard2_Load(object sender, EventArgs e)
        {
            load_world("..\\..\\..\\..\\worlds\\TrainYard\\TrainYard.3dstate");

            _3D.STATE_engine_set_background_color(135, 90, 60); //90, 60, 40

            load_model();

            create_the_weapon_fire_effect();

            m_master_shred=create_master_shred(); //the spliters that fly from the wall

            load_sounds();


            m_temp_camera = _3D.STATE_camera_create("test collision");


            uint crosshair_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\TrainYard\\extra_bitmaps\\crosshair", 0);
            m_crosshair_layer = _3D.STATE_layer_create(45, 45, 55, 55, crosshair_bmp, _3D.CONTROL_GLASS);


            //create_light();
            
            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;


            MessageBox.Show("Use the arrow keys to move.\r\n\nPress the Ctrl key to shoot (makes a mark)\r\n\r\nClick with the mouse on the 3D window to put a shooting mark\r\n\r\nThis demo demonstrates how to use the mouse to interact with the 3D world and how to get the polygon and location for a specific point on the 2D window ", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);


            ActiveControl = pictureBox_rendering_window;


        }

        bool load_world(string world_file_name)
        {

            int was_loaded_ok = _3D.STATE_engine_load_world(world_file_name, "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }

            m_camera = _3D.STATE_camera_get_default_camera();


            //Since it is a very small world we should be able to get close to objects
            //Every thing that is closer to the camera then the given number
            //will be cut out of the scene. See 3dstate.h for more details
            _3D.STATE_camera_set_distance_from_eye(m_camera, m_camera_projection_palne_distance);

            return (true);


        }

        void load_sounds()
        {
            m_shooting_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\shoot.wav", 0, _3D.SOUND_DISTANCE_DEFAULT);
            _3D.STATE_sound_set_volume(m_hit_wall_sound, 1000);

            for (int i = 0; i < 4; i++)
            {
                m_step_sounds[i] = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\step" + (i + 1).ToString() + ".wav", m_legs_obj, _3D.SOUND_DISTANCE_DEFAULT);
                _3D.STATE_sound_set_volume(m_step_sounds[i], 700);
            }

            m_fall_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\fall1.wav", m_legs_obj, _3D.SOUND_DISTANCE_DEFAULT);

            m_hit_wall_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\hit_wall.wav", m_legs_obj, _3D.SOUND_DISTANCE_DEFAULT);
            _3D.STATE_sound_set_volume(m_hit_wall_sound, 700);

            m_pain_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\pain1.wav", m_legs_obj, _3D.SOUND_DISTANCE_DEFAULT);

        }


        void check_shooting(double[] hitting_point, uint polygon_hit, uint obj_hit)
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.ControlKey) >= 0)
            {
                _3D.STATE_polygon_disable(m_fire_effect_polygon_front);
                _3D.STATE_polygon_disable(m_fire_effect_polygon_back);
                return;
            }
            add_shooting_mark(hitting_point, polygon_hit);

            play_shooting_sound();
            create_a_shooting_shred(hitting_point, polygon_hit, obj_hit);

            _3D.STATE_polygon_disable(m_crosshair_layer);
            _3D.STATE_polygon_enable(m_fire_effect_polygon_front);
            _3D.STATE_polygon_enable(m_fire_effect_polygon_back);
        }

        private void OnIdle(object sender, EventArgs e)
        {
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            if(checkBox_follow_model.Checked)
                put_camera_behind_object(m_legs_obj);
            else
                move_camera_according_to_keyboard_input();



            double[] hitting_point = new double[3];
            uint polygon_aimed;
            uint object_aimed;
            polygon_aimed = set_crosshair_location(hitting_point, out object_aimed);

            if (polygon_aimed!=0)
                check_shooting(hitting_point, polygon_aimed, object_aimed);


            move_object_according_to_keyboard_input();


            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, m_camera);



            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //on idle is called affter all events are processed, for some reasons (a bug in the .NET framework ?)
            //if there are no events, then OnIdle is not being called
            //therefor we crate a little onPaint event so that OnIdle is called
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...


        }

        //returns the polygon we are aiming at and the hitting point
        uint set_crosshair_location(double[] hitting_point, out uint object_aimed)
        {
            object_aimed=0;
          
            //can be done only with the base part of the model (the legs)
            uint obj = m_legs_obj;

           
            double[] location = new double[3];
            //note that this is the center of the legs
            _3D.STATE_object_get_location1(obj, location);
            location[2] += m_model_height/2;

            //move the object forward. (we want to have a line from the object to a far poinr in front of the object
            _3D.STATE_object_move(obj, _3D.OBJECT_SPACE, -1e6, 0, 0);

            double[] far_target = new double[3];
            _3D.STATE_object_get_location1(obj, far_target);
            far_target[2] += m_model_height/2; //since we took the center of the legs we need to put it heigher, aproximately the center of the upper body

            //move back
            _3D.STATE_object_move(obj, _3D.OBJECT_SPACE, 1e6, 0, 0);


            uint poly;
            _3D.STATE_engine_is_movement_possible(location, far_target, out poly, hitting_point, out object_aimed);

            if (poly == 0)
            {
                _3D.STATE_polygon_disable(m_crosshair_layer);
                return (0); ;
            }

            _3D.STATE_polygon_enable(m_crosshair_layer);
            int[] xy=new int[2];
            int rc=_3D.STATE_engine_3D_point_to_2D(hitting_point, xy);
            if (rc != 0)
            {
                _3D.STATE_polygon_disable(m_crosshair_layer);
                return (poly);
            }

            double width=_3D.STATE_engine_get_rendering_window_width();
            double height = _3D.STATE_engine_get_rendering_window_height();

            //convert the point to a 0-100 scale
            double x = Math.Round( (double)xy[0] * 100 / width);
            double y = Math.Round( (double)xy[1] * 100 / height);

            if (x > 100)
                x = 100;

            if (y > 100)
                y = 100;


            double[] start = { x - 5, y, m_camera_distance_from_object / m_camera_projection_palne_distance *1.3}; // so that the model head will be in front !!
            double[] end = { x + 5, y, m_camera_distance_from_object / m_camera_projection_palne_distance * 1.3 };
            _3D.STATE_layer_set_shape(m_crosshair_layer, start, end, 10, 10);

            return (poly);

        }

        void move_camera(double step)
        {
            double[] current_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, current_location);

            //move the camera so we can easily get the location where we want to go
            _3D.STATE_camera_move(m_camera, _3D.CAMERA_SPACE, step, 0, 0);
            double[] wanted_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, wanted_location);

            //we moved the camera but it could be that the way was blocked
            //so let move the camera back to it previous location
            //and do the movement again with collision detection
            _3D.STATE_camera_set_location1(m_camera, current_location); //return the camera back
            
            //now that we know the wanted location lets move the camera with collision detection
            _3D.STATE_camera_move_with_collision_detection(m_camera, wanted_location, m_walking_step * 10 );

        }

        
        
        void move_camera_according_to_keyboard_input()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.A) < 0)
                _3D.STATE_camera_rotate_z(m_camera, m_angles_to_turn, _3D.WORLD_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.D) < 0)
                _3D.STATE_camera_rotate_z(m_camera, -m_angles_to_turn, _3D.WORLD_SPACE);

            
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.W) < 0)
                move_camera(-m_walking_step);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Z) < 0)
                move_camera(m_walking_step);

        }
        

        //returns the maximum location that we can move from the current location to the wanted location
        //we have STATE_camera_move_with_collision_detection() but we dont have a similar STATE_object_move_with_collision_detection()
        //so we will put a camera in the place of the object to test collision        
        void get_possible_location(double []current_location,double[] wanted_location, double[] possible_location)
        {
            
            _3D.STATE_camera_set_location1(m_temp_camera, current_location);

            int rc=_3D.STATE_camera_move_with_collision_detection(m_temp_camera, wanted_location, m_walking_step * 30);

            //now the camera has moved to the maximum location possible on the line between the current location and the wanted location
            //so lets copy the location to wanted_location
            _3D.STATE_camera_get_location1(m_temp_camera, possible_location);
            
        }


        void move_object(double step)
        {
            //move the object up so if there are stairs we can climb them
            //and also make the colision detection in the height of the head and not the legs
            _3D.STATE_object_move(m_legs_obj, _3D.OBJECT_SPACE, 0, 0, m_model_height/2);
            
            double[] current_locatin = new double[3];
            _3D.STATE_object_get_location1(m_legs_obj, current_locatin);

            _3D.STATE_object_move(m_legs_obj, _3D.OBJECT_SPACE, step, 0, 0);
            double[] wanted_locatin = new double[3];
            _3D.STATE_object_get_location1(m_legs_obj, wanted_locatin);

            //we moved the object but we dont know if what we did is ok
            //maybe the way was blocked and we cant actually move. So lets check it
            double[] possible_location = new double[3];
            get_possible_location(current_locatin, wanted_locatin, possible_location);

            if(Vector.is_equal(possible_location,wanted_locatin, step*0.8)==false)
                play_hit_wall_sound();


            _3D.STATE_object_set_location1(m_legs_obj, possible_location);
            Vector.copy(current_locatin, possible_location);


            double min_height, max_height;
            _3D.STATE_object_get_bottom_height_above_ground(m_legs_obj,60, _3D.YES, out min_height, out max_height);
            min_height -= 5;
            if(min_height>0)
                _3D.STATE_object_move(m_legs_obj, _3D.WORLD_SPACE, 0, 0, -min_height);

            /*
            //now we have to check falling down.
            _3D.STATE_object_drop_down_fast(m_legs_obj, m_model_height, _3D.YES, _3D.NO);

            //we dropped the object, but the drop down function treats the object as if it has no volume (just a point)
            //se we need to do a better job
            _3D.STATE_object_get_location1(m_legs_obj, wanted_locatin);
            get_possible_location(current_locatin, wanted_locatin, possible_location);
            _3D.STATE_object_set_location1(m_legs_obj, possible_location);
            */



        }

        void move_object_according_to_keyboard_input()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0)
            {
                _3D.STATE_object_rotate_z(m_legs_obj, m_angles_to_turn, _3D.WORLD_SPACE);
                ActiveControl = pictureBox_rendering_window;
            }

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0)
            {
                _3D.STATE_object_rotate_z(m_legs_obj, -m_angles_to_turn, _3D.WORLD_SPACE);
                ActiveControl = pictureBox_rendering_window;
            }

            
            
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
            {
                set_animation_sequence(m_legs_obj, "WALK", false);
                move_object(-m_walking_step);
                make_step_sound(1);
                
                ActiveControl = pictureBox_rendering_window;

                return;
            }
            
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
            {
                set_animation_sequence(m_legs_obj, "WALK", true);
                move_object(m_walking_step);
                make_step_sound(-1);

                ActiveControl = pictureBox_rendering_window;
                return;
            }

            set_animation_sequence(m_legs_obj, "LANDB", false);



        }


        void put_camera_behind_object(uint obj)
        {
            double[] location = new double[3];
            _3D.STATE_object_get_location1(obj, location);

            location[2] += m_model_height / 2;

            //view abit from the side as well
            location[0] += m_model_height / 10;
            location[1] += m_model_height / 10;
         

            double[] wanted_location = Vector.copy(location);

            double[] direction = new double[3];
            _3D.STATE_object_get_direction1(obj, direction);

            Vector.mul_vec(direction, -m_camera_distance_from_object);
            Vector.add_vec(wanted_location, direction);

            uint poly, blocking_obj;
            double[] intersection = new double[3];
            int is_possible=_3D.STATE_engine_is_movement_possible(location, wanted_location, out poly, intersection, out blocking_obj);

            if (is_possible == _3D.YES)
                _3D.STATE_camera_set_location1(m_camera, wanted_location);
            else
            {
                Vector.sub_vec(intersection, location);
                Vector.mul_vec(intersection, 0.8);
                Vector.add_vec(intersection, location);
                _3D.STATE_camera_set_location1(m_camera, intersection);
            }


            _3D.STATE_camera_point_at(m_camera, location[0], location[1], location[2]);


        }

        //put the light source behind the object
        void set_light_location()
        {
            double[] location=new double[3];
            _3D.STATE_object_get_location1(m_legs_obj, location);

            double[] light_location=new double[3];
            _3D.STATE_object_move(m_legs_obj, _3D.OBJECT_SPACE, 100, 0, 0);
            _3D.STATE_object_get_location1(m_legs_obj, light_location);

            //now return the object back to its place
            _3D.STATE_object_set_location1(m_legs_obj, location);

            _3D.STATE_light2_set_location(m_light, light_location);


        }


        void create_light()
        {
            
            //m_light = _3D.STATE_light2_create_point_light("light_model", new double[] {0,0,0}, 200, 1);
            //_3D.STATE_light2_set_complex_attenuation(m_light, 1, 0, 0);

            m_light = _3D.STATE_light2_create_default_light("light_model");

            //_3D.STATE_light2_disable_specular_lighting(true);

            _3D.STATE_light2_set_ambient_color(m_light, new double[3] { 512, 512, 512 });
        }

        
        
        void add_some_ight_to_the_model2()
        {
            uint[] body_parts = { m_legs_obj, m_body_obj, m_head_obj, m_weapon_obj };
            byte[] rgb = { 128, 128, 128 };

            foreach (uint part in body_parts)
                _3D.STATE_object_set_light(part, rgb);

        }


        void load_model()
        {
            m_legs_obj = load_object("..\\..\\..\\..\\models\\human_character\\Lower.md3", "..\\..\\..\\..\\models\\human_character\\skins\\skin1.jpg", "..\\..\\..\\..\\models\\human_character\\animation.cfg", "LEGS_");
            m_body_obj = load_object("..\\..\\..\\..\\models\\human_character\\Upper.md3", "..\\..\\..\\..\\models\\human_character\\skins\\skin1.jpg", "..\\..\\..\\..\\models\\human_character\\animation.cfg", "TORSO_");
            m_head_obj = load_object("..\\..\\..\\..\\models\\human_character\\Head.md3", "model\\skins\\skin1.jpg", null, null);
            m_weapon_obj = load_object("..\\..\\..\\..\\models\\human_character\\railgun.md3", null, null, null);

            //_3D.STATE_object_set_scale(m_legs_obj, new double[3] { 2, 2, 2 });



            double length, width;
            _3D.STATE_object_get_dimensions(m_legs_obj, out length, out width, out m_model_height);
            m_model_height += 30; //since the first frame in the seqence is not the heighst one in length

            _3D.STATE_object_drop_down_fast(m_legs_obj, m_model_height, _3D.YES, _3D.NO);

            //connect the parts of the human model
            //Note that once th md3 objects are connected only the base object an be used (the legs) !!!
            _3D.STATE_object_md3_connect_models_automatic(m_legs_obj, m_body_obj);
            _3D.STATE_object_md3_connect_models_automatic(m_body_obj, m_head_obj);
            _3D.STATE_object_md3_connect_models_automatic(m_body_obj, m_weapon_obj);

            //set the lower body to walking sequence and an upper body to the Gesture sequence
            set_animation_sequence(m_legs_obj, "LANDB", false);
            set_animation_sequence(m_body_obj, "STAND", false);

            _3D.STATE_object_make_non_collisional(m_legs_obj);
            _3D.STATE_object_make_non_collisional(m_body_obj);
            _3D.STATE_object_make_non_collisional(m_head_obj);
            _3D.STATE_object_make_non_collisional(m_weapon_obj);

            _3D.STATE_group_calculate_points_normals(0);
        }

        class sequence_item
        {
            public string name;
            public uint seq_handle;

            public sequence_item(uint handle, string seq_name)
            {
                name = seq_name;
                seq_handle = handle;
            }

            public override string ToString()
            {
                if (name == null)
                    return ("no name");

                return name;
            }
        }



        //Set a specific animation sequence
        uint set_animation_sequence(uint obj, string sequence_name, bool play_backwards)
        {
            uint animation = _3D.STATE_object_get_3D_animation(obj);
            if (animation == 0)
                return (0);

            uint seq = _3D.STATE_3D_sequence_get_using_name(animation, sequence_name);
            if (seq == 0)
                return(0);

            _3D.STATE_object_set_3D_sequence(obj, seq, 0);

            _3D.STATE_3D_sequence_backwards_play(seq, play_backwards);

            return (seq);


        }



        //load a 3D object into the engine
        // These objects will be later rendered (painted on the screen) when _3D.STATE_engine_render() is called
        uint load_object(string file_name, string bmp_file, string cfg_file, string token)
        {

            uint obj;
            if (String.IsNullOrEmpty(cfg_file) == false)
                obj = _3D.STATE_object_md3_create_from_file(file_name, cfg_file, token);
            else
                obj = _3D.STATE_object_create_from_file(file_name);


            if (obj == 0)
                return (0);



            if (String.IsNullOrEmpty(bmp_file))
                return (obj);



            uint bmp = _3D.STATE_bitmap_load(bmp_file, -1);

            if (bmp == 0)
                return (obj);

            _3D.STATE_object_set_bitmap(obj, bmp, 0);


            return (obj);

        }

        private void checkBox_follow_model_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_follow_model.Checked)
                label_instructions.Visible = false;
            else
                label_instructions.Visible = true;
        }

        private void listBox_skins_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = listBox_skins.SelectedIndex;
            string image_file = m_skin_files[index];

            uint bmp = _3D.STATE_bitmap_load(image_file, -1);
            if (bmp == 0)
                return;

            _3D.STATE_object_set_bitmap(m_legs_obj, bmp, 0);
            _3D.STATE_object_set_bitmap(m_body_obj, bmp, 0);
            _3D.STATE_object_set_bitmap(m_head_obj, bmp, 0);

            ActiveControl = pictureBox_rendering_window;

        }
    
        //an example on how to create an animation from code, rather than loading it from file
        void create_fire_animation()
        {
	        //Create the fire animation
	        m_fire_animation=_3D.STATE_animation_create("shoot");
            uint frame1 = _3D.STATE_bitmap_load("../../../../Models/human_character/bitmaps/Mflash2.bmp", -1);
            uint frame2 = _3D.STATE_bitmap_load("../../../../Models/human_character/bitmaps/Mflash3.bmp", -1);
            uint frame3 = _3D.STATE_bitmap_load("../../../../Models/human_character/bitmaps/Mflash4.bmp", -1);

	        _3D.STATE_animation_add_bitmap(m_fire_animation, _3D.BITMAP_LIST_FRONT, frame1, -1);
            _3D.STATE_animation_add_bitmap(m_fire_animation, _3D.BITMAP_LIST_FRONT, frame2, -1);
            _3D.STATE_animation_add_bitmap(m_fire_animation, _3D.BITMAP_LIST_FRONT, frame3, -1);

	        uint[] times={1,1,1};
	        _3D.STATE_animation_set_times(m_fire_animation, times, 3);
        }


        //Finds a point at the end of the weapon. We need it to connect
        //the polygon with the fire animation.
        void find_a_point_at_the_end_of_the_weapon(double[] result, out uint edge_point)
        {
            edge_point = 0;
            
            double min_point = 999999999;
            result[0] = 0; result[1] = 0; result[2] = 0;

            for (uint poly = _3D.STATE_object_get_first_polygon(m_weapon_obj); poly != 0; poly = _3D.STATE_object_get_next_polygon(m_weapon_obj, poly))
            {
                for (uint point = _3D.STATE_polygon_get_first_point(poly); point != 0; point = _3D.STATE_point_get_next_point(point))
                {
                    double[] xyz = new double[3];
                    _3D.STATE_point_get_xyz(point, xyz);
                    if (xyz[0] < min_point)
                    {
                        Vector.copy(result, xyz);
                        min_point = xyz[0];
                        edge_point = point;
                    }
                }
            }

        }

        
        //Creates the polygon that is used the create the fire effect 
        //when shooting
        void create_the_weapon_fire_effect()
        {
            create_fire_animation();

	        m_fire_effect_polygon_front=_3D.STATE_polygon_create();

	        //All the code below is simply to get the edge of the gun so we know where to
	        //put the polygon with the fire effect
	        double[] edge_point=new double[3];
            find_a_point_at_the_end_of_the_weapon(edge_point, out m_weapon_edge_point);

	        //Now lets build a square around the edge point
	        double[] point1={0,0,0,	0,0,  100};
	        double[] point2={0,0,0,	1,0,  100};
	        double[] point3={0,0,0,	1,1,  100};
	        double[] point4={0,0,0,	0,1,  100};
	        //double size=VIEWER_SIZE/10;
	        double size=30;
	        Vector.set(point1, edge_point[0], edge_point[1]-size, edge_point[2]+size-5); //the -5 is because the edge point we get is a bit too high, we need the cemter of the barrel of the gun
	        Vector.set(point2, edge_point[0], edge_point[1]+size, edge_point[2]+size-5);
	        Vector.set(point3, edge_point[0], edge_point[1]+size, edge_point[2]-size-5);
	        Vector.set(point4, edge_point[0], edge_point[1]-size, edge_point[2]-size-5);

        	
	        _3D.STATE_polygon_add_point(m_fire_effect_polygon_front, point1);
	        _3D.STATE_polygon_add_point(m_fire_effect_polygon_front, point2);
	        _3D.STATE_polygon_add_point(m_fire_effect_polygon_front, point3);
	        _3D.STATE_polygon_add_point(m_fire_effect_polygon_front, point4);
        	
	        _3D.STATE_object_add_polygon(m_weapon_obj, m_fire_effect_polygon_front);

	        _3D.STATE_polygon_set_animation(m_fire_effect_polygon_front, m_fire_animation);
	        _3D.STATE_polygon_set_translucent(m_fire_effect_polygon_front,_3D.LIGHT_SOURCE_GLASS);

	        //For the time being we will disable this polygon
	        //When shooting we will enable it again
	        _3D.STATE_polygon_disable(m_fire_effect_polygon_front);
	        //_3D.STATE_polygon_set_rotated(m_fire_effect_polygon_front, YES);

            m_fire_effect_polygon_back = _3D.STATE_polygon_duplicate(m_fire_effect_polygon_front);
            _3D.STATE_polygon_flip_visible_side(m_fire_effect_polygon_back);
            _3D.STATE_object_add_polygon(m_weapon_obj, m_fire_effect_polygon_back);
            _3D.STATE_polygon_disable(m_fire_effect_polygon_back);


        }

        
        //An Example on how to create an object from ode rather than
        //load it from file.
        //
        //When we shoot something, tiny fractions of that objects will fall down.
        //Here we create one such piece\fraction\splinter\shred
        //All the other shreds will be a duplication of the master shred
        uint create_master_shred()
        {
	        uint obj=_3D.STATE_object_create("shred");

	        uint poly=_3D.STATE_polygon_create();
        	
	        double[] point1={0,0,0,	0,0,  100};
	        double[] point2={0,0,0,	1,0,  100};
	        double[] point3={0,0,0,	1,1,  100};
	        double[] point4={0,0,0,	0,1,  100};
	        double size=3; //shred size
	        Vector.set(point1, 0, -size, size);
	        Vector.set(point2, 0, size, size);
	        Vector.set(point3, 0, size, -size);
	        Vector.set(point4, 0, -size, -size);
	        _3D.STATE_polygon_add_point(poly, point1);
	        _3D.STATE_polygon_add_point(poly, point2);
	        _3D.STATE_polygon_add_point(poly, point3);
	        _3D.STATE_polygon_add_point(poly, point4);
        	
	        _3D.STATE_object_add_polygon(obj, poly);

	        _3D.STATE_polygon_set_rotated(poly, _3D.YES);

            m_shooting_mark_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\TrainYard\\extra_bitmaps\\smark", 0);


            _3D.STATE_polygon_set_bitmap_fill(poly, m_shooting_mark_bmp);

	        _3D.STATE_polygon_set_translucent(poly, _3D.DARK_FILTER_GLASS );

	        _3D.STATE_object_disable(obj);
        	
	        return(obj);

        }
        
        //When we shoot at the wall, pieces\fractions of the wall fall down.
        //Here we create one piece\fraction and we make sure it falls down
        void create_a_shooting_shred(double[] target, uint target_polygon, uint target_object)
        {
        	
	        if(target_polygon==0) return;
        	
	        if(target_object!=0) 
		        return; //Most likely that we hit a shred that we have created in the previous calls
        	
	        uint shred=_3D.STATE_object_duplicate(m_master_shred, _3D.NO); 

	        //Set the location of the shred in front of the target polygon
	        double[] normal=new double[4];
	        _3D.STATE_polygon_get_plane(target_polygon, normal);
	        Vector.mul_vec(normal, 5);
	        Vector.add_vec(target, normal);
	        _3D.STATE_object_set_location1(shred, target);

	        _3D.STATE_object_set_chase_type(shred,_3D.CHASE_PHYSICS);
        	
	        _3D.STATE_object_set_max_speed(shred, 60); 
        	
	        
            double[] speed={m_rnd.NextDouble()-0.5,m_rnd.NextDouble()-0.5,m_rnd.NextDouble()};

	        //Set the direction
	        _3D.STATE_object_set_speed(shred,speed);

	        //Set the size of the speed
	        _3D.STATE_object_set_absolute_speed(shred,6);

	        _3D.STATE_object_set_friction(shred, 0);

	        _3D.STATE_object_set_elasticity(shred,0.4);

	        double[] force={0,0,-1};
	        _3D.STATE_object_set_force(shred,force);

	        //order the engine to delete this object after 5 seconds
	        _3D.STATE_object_set_event(shred, 5000, _3D.STATE_DELETE);

        }

        DateTime m_last_shooting = DateTime.Now;


        void add_shooting_mark(double[] xyz, uint poly)
        {
            TimeSpan ts = DateTime.Now - m_last_shooting;
            if (ts.TotalSeconds < 0.3)
                return;


            if (poly == 0)
                return;


            int[] rgb = { 255, 255, 255 };
            double[] direction = { 0, 0, 1 };
            uint patch = _3D.STATE_polygon_add_patch_easy(poly, xyz, 5, direction, m_shooting_mark_bmp, rgb);

            _3D.STATE_polygon_set_translucent(patch, _3D.DARK_FILTER_GLASS);
            _3D.STATE_polygon_set_second_bitmap(patch, 0);

            //_3D.STATE_layer_create(0, 0, 20, 20, m_shooting_mark_bmp, 0);

            _3D.STATE_sound_play(m_shooting_sound, _3D.SOUND_NO_LOOP);


            m_last_shooting = DateTime.Now;

        }
        

        #region sounds

        
        void play_hit_wall_sound()
        {
            if(_3D.STATE_sound_is_sound_playing(m_hit_wall_sound)==_3D.NO)
                _3D.STATE_sound_play(m_hit_wall_sound, _3D.SOUND_NO_LOOP);

        }

        int m_step_sound_index = 0;
        DateTime m_last_time_we_made_a_step_sound;
        
        void make_step_sound(int direction)
        {
            DateTime now = DateTime.Now;
            TimeSpan time_passed = now - m_last_time_we_made_a_step_sound;

            if (time_passed.TotalSeconds < 0.5)
                return;

            _3D.STATE_sound_play(m_step_sounds[m_step_sound_index], _3D.SOUND_NO_LOOP);

            m_last_time_we_made_a_step_sound = now;

            m_step_sound_index += direction;

            if (m_step_sound_index < 0)
            {
                m_step_sound_index = 3;
                return;
            }


            if (m_step_sound_index >=4)
            {
                m_step_sound_index = 0;
                return;
            }


        }

        void play_shooting_sound()
        {
            if (_3D.STATE_sound_is_sound_playing(m_shooting_sound) == _3D.NO)
                _3D.STATE_sound_play(m_shooting_sound, _3D.SOUND_NO_LOOP);

        }

        #endregion

        private void checkBox_light_CheckedChanged(object sender, EventArgs e)
        {

            if (checkBox_light.Checked)
            {
                if (m_light == 0)
                    create_light();

                _3D.STATE_light2_disable_all_lighting(false);


                _3D.STATE_light2_enable(m_light);
                return;
            }

            //_3D.STATE_light2_disable(m_light);
            _3D.STATE_light2_disable_all_lighting(true);


        }



    }
}
